package hu.bme.mit.train.cc.eoclCG;

import hu.bme.mit.train.util.BenchmarkConfig;
import hu.bme.mit.train.util.BenckmarkResult;
import hu.bme.mit.train.util.CcBenchmark;
import hu.bme.mit.train.util.EMFModification;
import hu.bme.mit.train.util.FileBroker;
import hu.bme.mit.train.util.Util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import Concept.IndividualContainer;
import Concept.Route;

public class TestCaseSignalNeighbor_EoclCG implements CcBenchmark {

	public static Collection<Object[]> data() {
		Object[][] data = new Object[][] { 
				{1}, {2}, {4}, {8}, {16}, {32}, {64}, {128}, {256}, {512}
		};
		return Arrays.asList(data);
	}
	
	private String fileName;
	private BenckmarkResult bmr;
	private BenchmarkConfig bc;
	private List<Route> invalids;
	private List<Route> allRoutes;
	private static final boolean onlyForRoutes = true;

	private IndividualContainer pack;

	@Override
	public void init(BenchmarkConfig bc) {
		bmr = new BenckmarkResult(this.getClass().getSimpleName());
		bmr.setSize(bc.getModelSize());
		this.fileName = "models/testBig" + bc.getModelVariant() + bc.getModelSize() + ".concept";
		bmr.setFileName(this.fileName);
		
		System.err.println(org.eclipse.ocl.examples.pivot.PivotTables.class);

		this.bc = bc;
		Util.runGC();
		if (bc.isBenchmarkMode()) {
			Util.freeCache(bc);
		}
		
	}
	@Override
	public void loadModel() {
		bmr.startStopper();
		URI resourceURI = FileBroker.getBigTestModelFile(fileName);
		ResourceSet resourceSet = new ResourceSetImpl();
		Resource resource = resourceSet.getResource(resourceURI, true);

		if (resource.getContents().size() > 0 && resource.getContents().get(0) instanceof IndividualContainer) {
			pack = (IndividualContainer) resource.getContents().get(0);
			if (onlyForRoutes) {
				allRoutes = new ArrayList<Route>();
				for (Object thing : pack.getContains())
					if (thing instanceof Route) 
						allRoutes.add((Route) thing);
			}
		}
		
		bmr.setReadTime();
	}
	@Override
	public void check() {
		bmr.startStopper();
		
		// check instance model
		invalids = new ArrayList<Route>();

        for (Object eo : onlyForRoutes ? allRoutes : pack.getContains()) {
			if (eo instanceof Route) {
				Route route = (Route) eo;
				boolean result = route.signalNeighbor();
				if (result) {
					invalids.add(route);
				}
			}
		}
		
		/*for (Object segment : invalids) {
			System.out.println(segment);
		}*/
		
		bmr.addInvalid(invalids.size());
		bmr.addCheckTime();
	}
	@Override
	public void modifyUserScenario() {
		EMFModification.modifyEMFmodelSignalNeighbor(pack, bmr, Util.calcModify(bc, bc.getModifyAmountRouteSensor(), bmr));
	}
	@Override
	public void modifyModelXFormScenario() {
		EMFModification.modifyEMFmodelSignalNeighborRepair(pack, bmr, Util.calcModify(bc, bc.getModifyAmountRouteSensor(), bmr), invalids);
	}
	@Override
	public void measureMemory() {
		Util.runGC();
		bmr.addMemoryBytes(Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory());
	}
	@Override
	public void destroy() {
	}
	@Override
	public BenckmarkResult getBMR() {
		return bmr;
	}

}
