package hu.bme.mit.train.cc.eoclPivot;

import hu.bme.mit.train.common.RuntimeConstants;
import hu.bme.mit.train.util.BenchmarkConfig;
import hu.bme.mit.train.util.BenckmarkResult;
import hu.bme.mit.train.util.CcBenchmark;
import hu.bme.mit.train.util.EMFModification;
import hu.bme.mit.train.util.FileBroker;
import hu.bme.mit.train.util.Util;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.ocl.examples.pivot.ExpressionInOCL;
import org.eclipse.ocl.examples.pivot.Type;
import org.eclipse.ocl.examples.pivot.manager.MetaModelManager;
import org.eclipse.ocl.examples.pivot.model.OCLstdlib;
import org.eclipse.ocl.examples.xtext.oclinecore.OCLinEcoreStandaloneSetup;

import Concept.ConceptPackage;
import Concept.IndividualContainer;
import Concept.Sensor;

public class TestCaseSwitchSet_EoclPivot implements CcBenchmark {

	public static Collection<Object[]> data() {
		Object[][] data = new Object[][] { 
				{1}, {2}, {4}, {8}, {16}, {32}, {64}, {128}, {256}, {512}
		};
		return Arrays.asList(data);
	}
	
	private String fileName;
	private BenckmarkResult bmr;
	private BenchmarkConfig bc;
	private List<Sensor> invalids;

	// context: Segment
	private String oclConstraint;

	private IndividualContainer pack;
	private ExpressionInOCL query;
	private org.eclipse.ocl.examples.pivot.OCL oclPivot;

	@Override
	public void init(BenchmarkConfig bc) {
		bmr = new BenckmarkResult(this.getClass().getSimpleName());
		bmr.setSize(bc.getModelSize());
		this.fileName = "models/testBig" + bc.getModelVariant() + bc.getModelSize() + ".concept";
		bmr.setFileName(this.fileName);
		
		this.bc = bc;
		Util.runGC();
		if (bc.isBenchmarkMode()) {
			Util.freeCache(bc);
		}
	}
	@Override
	public void loadModel() {
		bmr.startStopper();
		
		OCLinEcoreStandaloneSetup.doSetup();
        OCLstdlib.install(); 
		
		URI resourceURI = FileBroker.getBigTestModelFile(fileName);
		ResourceSet resourceSet = new ResourceSetImpl();
		org.eclipse.ocl.examples.pivot.delegate.OCLDelegateDomain.initialize(resourceSet); 
		Resource resource = resourceSet.getResource(resourceURI, true);

		if (resource.getContents().size() > 0 && resource.getContents().get(0) instanceof IndividualContainer) {
			pack = (IndividualContainer) resource.getContents().get(0);
		}
		
		// init OCL
		try {
			oclConstraint = Util.readFile(RuntimeConstants.projectPath + "/models/query/ocl/SwitchSet.ocl");
		} catch (IOException e) {
			e.printStackTrace();
		}

		//oclConstraint = "not self.Sensor_trackElement->forAll(te|te.oclAsType(Switch).Switch_switchPosition->forAll(sp| sp.SwitchPosition_route.Route_routeDefinition->includes(self)))";
		oclPivot = org.eclipse.ocl.examples.pivot.OCL.newInstance();
		org.eclipse.ocl.examples.pivot.helper.OCLHelper helper = oclPivot.createOCLHelper();
		MetaModelManager metaModelManager = oclPivot.getMetaModelManager();
		Type pivotType = (Type) metaModelManager.getType(ConceptPackage.eINSTANCE.getSensor());
		query = null;
		try {
			helper.setContext(pivotType);
			query = helper.createQuery(oclConstraint);
		} catch (org.eclipse.ocl.examples.pivot.ParserException e) {
			e.printStackTrace();
		}

		
		bmr.setReadTime();
	}
	@Override
	public void check() {
		bmr.startStopper();
		
		// check instance model
		invalids = new ArrayList<Sensor>();
		if (query != null) {
			for (Object o : pack.getContains()) {
				Object obj = oclPivot.evaluate(o, query);
				if (obj == Boolean.TRUE) {
					invalids.add((Sensor) o);
				}
			}
		}

		/*for (Object segment : invalids) {
			System.out.println(segment);
		}*/
		
		bmr.addInvalid(invalids.size());
		bmr.addCheckTime();
	}
	@Override
	public void modifyUserScenario() {
		EMFModification.modifyEMFmodelSwitchSet(pack, bmr, Util.calcModify(bc, bc.getModifyAmountRouteSensor(), bmr));
	}
	@Override
	public void modifyModelXFormScenario() {
		// throw new RuntimeException(new OperationNotSupportedException());
	}
	@Override
	public void measureMemory() {
		Util.runGC();
		bmr.addMemoryBytes(Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory());
	}
	@Override
	public void destroy() {
		oclPivot.dispose();
		OCLstdlib.uninstall();
		OCLinEcoreStandaloneSetup.doTearDown();
	}
	@Override
	public BenckmarkResult getBMR() {
		return bmr;
	}

}
