package hu.bme.mit.train.cc.oclImpact;

import hu.bme.mit.train.common.RuntimeConstants;
import hu.bme.mit.train.constrintcheck.eoclia.OCLIAUtils;
import hu.bme.mit.train.constrintcheck.eoclia.ResultAdapter;
import hu.bme.mit.train.util.BenchmarkConfig;
import hu.bme.mit.train.util.BenckmarkResult;
import hu.bme.mit.train.util.CcBenchmark;
import hu.bme.mit.train.util.EMFModification;
import hu.bme.mit.train.util.FileBroker;
import hu.bme.mit.train.util.FileBroker2;
import hu.bme.mit.train.util.Util;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.ocl.ecore.OCL;
import org.eclipse.ocl.ecore.OCLExpression;
import org.eclipse.ocl.ecore.OCL.Helper;
import org.eclipse.ocl.examples.eventmanager.EventFilter;
import org.eclipse.ocl.examples.eventmanager.EventManager;
import org.eclipse.ocl.examples.eventmanager.EventManagerFactory;
import org.eclipse.ocl.examples.impactanalyzer.ImpactAnalyzer;
import org.eclipse.ocl.examples.impactanalyzer.ImpactAnalyzerFactory;
import org.eclipse.ocl.examples.impactanalyzer.util.OCLFactory;

import Concept.ConceptPackage;
import Concept.IndividualContainer;
import Concept.Sensor;

public class TestCaseRouteSensor_Eoclia implements CcBenchmark {

	public static Collection<Object[]> data() {
		Object[][] data = new Object[][] { 
				{1}, {2}, {4}, {8}, {16}, {32}, {64}, {128}, {256}, {512}
		};
		return Arrays.asList(data);
	}
	
	private String fileName;
	private BenckmarkResult bmr;
	private BenchmarkConfig bc;
	private List<Sensor> invalids;
	private IndividualContainer pack;
	private OCL ocl;
	private OCLExpression query; 
	private String oclConstraint;
	private ResultAdapter<Sensor> results;
	private EventManager eventManager;
	private ResourceSet resourceSet;
	private boolean isBatch;


	@Override
	public void init(BenchmarkConfig bc) {
		bmr = new BenckmarkResult(this.getClass().getSimpleName());
		bmr.setSize(bc.getModelSize());
		this.fileName = "models/testBig" + bc.getModelVariant() + bc.getModelSize() + ".concept";
		bmr.setFileName(this.fileName);
		
		this.bc = bc;
		Util.runGC();
		if (bc.isBenchmarkMode()) {
			Util.freeCache(bc);
		}
		
		isBatch = true;
	}
	@Override
	public void loadModel() {
		bmr.startStopper();
		// load an EMF instance as a resource
		URI resourceURI = FileBroker2.getEMFUri(bc.getInstanceModelPath() + "/" + fileName);
		resourceSet = new ResourceSetImpl();

		// build OCL constraint
		try {
			oclConstraint = Util.readFile(RuntimeConstants.projectPath + "/models/query/ocl/RouteSensor.ocl");
		} catch (IOException e) {
			e.printStackTrace();
		}
		ocl = OCL.newInstance();
		Helper helper = ocl.createOCLHelper();
		query = null;
		try {
			// set context
			helper.setContext(ConceptPackage.eINSTANCE.getSensor());
			query = helper.createQuery(oclConstraint);
		} catch (org.eclipse.ocl.ParserException e) {
			e.printStackTrace();
		}

		// load the EMF resource
		Resource resource = resourceSet.getResource(resourceURI, true);

		if (resource.getContents().size() > 0 && resource.getContents().get(0) instanceof IndividualContainer) {
			pack = (IndividualContainer) resource.getContents().get(0);
		}
		bmr.setReadTime();
	}
	static boolean firstCheck = true;
	@Override
	public void check() {
		bmr.startStopper();
		if (firstCheck) {
		if (isBatch) {
			invalids = new ArrayList<Sensor>();
	        if (query != null) {
	            for (Object eObject : pack.getContains()) {
	                Object result = ocl.evaluate(eObject, query);
	                if (result == Boolean.TRUE) {
	                    invalids.add((Sensor) eObject);
	                }
	            }
	        }
	        
			// create impact analyzer based on the OCL expression
			final ImpactAnalyzer impactAnalyzer = ImpactAnalyzerFactory.INSTANCE.createImpactAnalyzer(
					query,  // the expression to re-evaluate incrementally
					true,  // whether to re-evaluate when new context objects appear
					OCLFactory.getInstance());
			// attach a simple result adapter to watch possible context items
			results = new ResultAdapter<Sensor>(impactAnalyzer, ocl, query, invalids);
			EventFilter filter = impactAnalyzer.createFilterForExpression();
			eventManager = EventManagerFactory.eINSTANCE.getEventManagerFor(resourceSet);
			eventManager.subscribe(filter, results);
			
			isBatch = false;
		} else {
			invalids = new ArrayList<Sensor>(results.getInvalids());
		}

        /*for (Segment segment : invalids) {
            System.out.println(segment);
        }*/
		if (bc.getScenarioVariant() >= 1) firstCheck = false;
		}
        
		bmr.addInvalid(invalids.size());
		bmr.addCheckTime();
	}
	@Override
	public void modifyUserScenario() {
		if (bc.getScenarioVariant() < 1)
			EMFModification.modifyEMFmodelRouteSensor(pack, bmr, Util.calcModify(bc, bc.getModifyAmountRouteSensor(), bmr));
	}
	@Override
	public void modifyModelXFormScenario() {
		if (bc.getScenarioVariant() < 1)
			EMFModification.modifyEMFmodelRouteSensorRepair(pack, bmr, Util.calcModify(bc, bc.getModifyAmountRouteSensor(), bmr), invalids);
	}
	@Override
	public void measureMemory() {
		Util.runGC();
		bmr.addMemoryBytes(Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory());
	}
	@Override
	public void destroy() {
		eventManager.unsubscribe(results);
		eventManager.removeFromObservedResourceSets(resourceSet);
		ocl.dispose();
		
		OCLIAUtils.oclIAHack(bc, this.getClass().getName() + "_" + bc.getCurrentScenario(), bmr);
	}
	@Override
	public BenckmarkResult getBMR() {
		return bmr;
	}

}
