package hu.bme.mit.train.constraintcheck.incquery;

import Concept.Route;
import Concept.Sensor;
import Concept.Switch;
import Concept.SwitchPosition;
import hu.bme.mit.train.constraintcheck.incquery.util.RouteSensorQuerySpecification;
import java.util.Arrays;
import java.util.List;
import org.eclipse.incquery.runtime.api.IPatternMatch;
import org.eclipse.incquery.runtime.api.impl.BasePatternMatch;
import org.eclipse.incquery.runtime.exception.IncQueryException;

/**
 * Pattern-specific match representation of the hu.bme.mit.train.constraintcheck.incquery.routeSensor pattern,
 * to be used in conjunction with {@link RouteSensorMatcher}.
 * 
 * <p>Class fields correspond to parameters of the pattern. Fields with value null are considered unassigned.
 * Each instance is a (possibly partial) substitution of pattern parameters,
 * usable to represent a match of the pattern in the result of a query,
 * or to specify the bound (fixed) input parameters when issuing a query.
 * 
 * @see RouteSensorMatcher
 * @see RouteSensorProcessor
 * 
 */
@SuppressWarnings("all")
public abstract class RouteSensorMatch extends BasePatternMatch {
  private Sensor fSen;
  
  private Switch fSw;
  
  private SwitchPosition fSp;
  
  private Route fR;
  
  private static List<String> parameterNames = makeImmutableList("Sen", "Sw", "Sp", "R");
  
  private RouteSensorMatch(final Sensor pSen, final Switch pSw, final SwitchPosition pSp, final Route pR) {
    this.fSen = pSen;
    this.fSw = pSw;
    this.fSp = pSp;
    this.fR = pR;
    
  }
  
  @Override
  public Object get(final String parameterName) {
    if ("Sen".equals(parameterName)) return this.fSen;
    if ("Sw".equals(parameterName)) return this.fSw;
    if ("Sp".equals(parameterName)) return this.fSp;
    if ("R".equals(parameterName)) return this.fR;
    return null;
    
  }
  
  public Sensor getSen() {
    return this.fSen;
    
  }
  
  public Switch getSw() {
    return this.fSw;
    
  }
  
  public SwitchPosition getSp() {
    return this.fSp;
    
  }
  
  public Route getR() {
    return this.fR;
    
  }
  
  @Override
  public boolean set(final String parameterName, final Object newValue) {
    if (!isMutable()) throw new java.lang.UnsupportedOperationException();
    if ("Sen".equals(parameterName) ) {
    	this.fSen = (Concept.Sensor) newValue;
    	return true;
    }
    if ("Sw".equals(parameterName) ) {
    	this.fSw = (Concept.Switch) newValue;
    	return true;
    }
    if ("Sp".equals(parameterName) ) {
    	this.fSp = (Concept.SwitchPosition) newValue;
    	return true;
    }
    if ("R".equals(parameterName) ) {
    	this.fR = (Concept.Route) newValue;
    	return true;
    }
    return false;
    
  }
  
  public void setSen(final Sensor pSen) {
    if (!isMutable()) throw new java.lang.UnsupportedOperationException();
    this.fSen = pSen;
    
  }
  
  public void setSw(final Switch pSw) {
    if (!isMutable()) throw new java.lang.UnsupportedOperationException();
    this.fSw = pSw;
    
  }
  
  public void setSp(final SwitchPosition pSp) {
    if (!isMutable()) throw new java.lang.UnsupportedOperationException();
    this.fSp = pSp;
    
  }
  
  public void setR(final Route pR) {
    if (!isMutable()) throw new java.lang.UnsupportedOperationException();
    this.fR = pR;
    
  }
  
  @Override
  public String patternName() {
    return "hu.bme.mit.train.constraintcheck.incquery.routeSensor";
    
  }
  
  @Override
  public List<String> parameterNames() {
    return RouteSensorMatch.parameterNames;
    
  }
  
  @Override
  public Object[] toArray() {
    return new Object[]{fSen, fSw, fSp, fR};
    
  }
  
  @Override
  public String prettyPrint() {
    StringBuilder result = new StringBuilder();
    result.append("\"Sen\"=" + prettyPrintValue(fSen) + ", ");
    result.append("\"Sw\"=" + prettyPrintValue(fSw) + ", ");
    result.append("\"Sp\"=" + prettyPrintValue(fSp) + ", ");
    result.append("\"R\"=" + prettyPrintValue(fR));
    return result.toString();
    
  }
  
  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((fSen == null) ? 0 : fSen.hashCode());
    result = prime * result + ((fSw == null) ? 0 : fSw.hashCode());
    result = prime * result + ((fSp == null) ? 0 : fSp.hashCode());
    result = prime * result + ((fR == null) ? 0 : fR.hashCode());
    return result;
    
  }
  
  @Override
  public boolean equals(final Object obj) {
    if (this == obj)
    	return true;
    if (!(obj instanceof RouteSensorMatch)) { // this should be infrequent
    	if (obj == null)
    		return false;
    	if (!(obj instanceof IPatternMatch))
    		return false;
    	IPatternMatch otherSig  = (IPatternMatch) obj;
    	if (!specification().equals(otherSig.specification()))
    		return false;
    	return Arrays.deepEquals(toArray(), otherSig.toArray());
    }
    RouteSensorMatch other = (RouteSensorMatch) obj;
    if (fSen == null) {if (other.fSen != null) return false;}
    else if (!fSen.equals(other.fSen)) return false;
    if (fSw == null) {if (other.fSw != null) return false;}
    else if (!fSw.equals(other.fSw)) return false;
    if (fSp == null) {if (other.fSp != null) return false;}
    else if (!fSp.equals(other.fSp)) return false;
    if (fR == null) {if (other.fR != null) return false;}
    else if (!fR.equals(other.fR)) return false;
    return true;
  }
  
  @Override
  public RouteSensorQuerySpecification specification() {
    try {
    	return RouteSensorQuerySpecification.instance();
    } catch (IncQueryException ex) {
     	// This cannot happen, as the match object can only be instantiated if the query specification exists
     	throw new IllegalStateException	(ex);
    }
    
  }
  
  @SuppressWarnings("all")
  static final class Mutable extends RouteSensorMatch {
    Mutable(final Sensor pSen, final Switch pSw, final SwitchPosition pSp, final Route pR) {
      super(pSen, pSw, pSp, pR);
      
    }
    
    @Override
    public boolean isMutable() {
      return true;
    }
  }
  
  
  @SuppressWarnings("all")
  static final class Immutable extends RouteSensorMatch {
    Immutable(final Sensor pSen, final Switch pSw, final SwitchPosition pSp, final Route pR) {
      super(pSen, pSw, pSp, pR);
      
    }
    
    @Override
    public boolean isMutable() {
      return false;
    }
  }
  
}
