package hu.bme.mit.train.constraintcheck.incquery;

import Concept.Route;
import Concept.Sensor;
import Concept.Switch;
import Concept.SwitchPosition;
import hu.bme.mit.train.constraintcheck.incquery.RouteSensorMatch;
import hu.bme.mit.train.constraintcheck.incquery.util.RouteSensorQuerySpecification;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.incquery.runtime.api.IMatchProcessor;
import org.eclipse.incquery.runtime.api.IQuerySpecification;
import org.eclipse.incquery.runtime.api.IncQueryEngine;
import org.eclipse.incquery.runtime.api.impl.BaseMatcher;
import org.eclipse.incquery.runtime.exception.IncQueryException;
import org.eclipse.incquery.runtime.matchers.tuple.Tuple;
import org.eclipse.incquery.runtime.rete.misc.DeltaMonitor;

/**
 * Generated pattern matcher API of the hu.bme.mit.train.constraintcheck.incquery.routeSensor pattern,
 * providing pattern-specific query methods.
 * 
 * <p>Use the pattern matcher on a given model via {@link #on(IncQueryEngine)},
 * e.g. in conjunction with {@link IncQueryEngine#on(Notifier)}.
 * 
 * <p>Matches of the pattern will be represented as {@link RouteSensorMatch}.
 * 
 * <p>Original source:
 * <code><pre>
 * // expected result: 90
 * 
 * pattern routeSensor(Sen, Sw, Sp, R) = {
 * 	Route(R);
 * 	SwitchPosition(Sp);
 * 	Switch(Sw);
 * 	Sensor(Sen);
 * 	
 * 	Route.Route_switchPosition(R, Sp);
 * 	SwitchPosition.SwitchPosition_switch(Sp, Sw);
 * 	Trackelement.TrackElement_sensor(Sw, Sen);
 * 	
 * 	neg find head(Sen, R);	
 * }
 * </pre></code>
 * 
 * @see RouteSensorMatch
 * @see RouteSensorProcessor
 * @see RouteSensorQuerySpecification
 * 
 */
@SuppressWarnings("all")
public class RouteSensorMatcher extends BaseMatcher<RouteSensorMatch> {
  /**
   * @return the singleton instance of the query specification of this pattern
   * @throws IncQueryException if the pattern definition could not be loaded
   * 
   */
  public static IQuerySpecification<RouteSensorMatcher> querySpecification() throws IncQueryException {
    return RouteSensorQuerySpecification.instance();
  }
  
  /**
   * Initializes the pattern matcher within an existing EMF-IncQuery engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing EMF-IncQuery engine in which this matcher will be created.
   * @throws IncQueryException if an error occurs during pattern matcher creation
   * 
   */
  public static RouteSensorMatcher on(final IncQueryEngine engine) throws IncQueryException {
    // check if matcher already exists
    RouteSensorMatcher matcher = engine.getExistingMatcher(querySpecification());
    if (matcher == null) {
    	matcher = new RouteSensorMatcher(engine);
    	// do not have to "put" it into engine.matchers, reportMatcherInitialized() will take care of it
    }
    return matcher;
  }
  
  private final static int POSITION_SEN = 0;
  
  private final static int POSITION_SW = 1;
  
  private final static int POSITION_SP = 2;
  
  private final static int POSITION_R = 3;
  
  /**
   * Initializes the pattern matcher over a given EMF model root (recommended: Resource or ResourceSet).
   * If a pattern matcher is already constructed with the same root, only a light-weight reference is returned.
   * The scope of pattern matching will be the given EMF model root and below (see FAQ for more precise definition).
   * The match set will be incrementally refreshed upon updates from this scope.
   * <p>The matcher will be created within the managed {@link IncQueryEngine} belonging to the EMF model root, so
   * multiple matchers will reuse the same engine and benefit from increased performance and reduced memory footprint.
   * @param emfRoot the root of the EMF containment hierarchy where the pattern matcher will operate. Recommended: Resource or ResourceSet.
   * @throws IncQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(IncQueryEngine)} instead, e.g. in conjunction with {@link IncQueryEngine#on(Notifier)}
   * 
   */
  @Deprecated
  public RouteSensorMatcher(final Notifier emfRoot) throws IncQueryException {
    this(IncQueryEngine.on(emfRoot));
  }
  
  /**
   * Initializes the pattern matcher within an existing EMF-IncQuery engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing EMF-IncQuery engine in which this matcher will be created.
   * @throws IncQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(IncQueryEngine)} instead
   * 
   */
  @Deprecated
  public RouteSensorMatcher(final IncQueryEngine engine) throws IncQueryException {
    super(engine, querySpecification());
  }
  
  /**
   * Returns the set of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pSen the fixed value of pattern parameter Sen, or null if not bound.
   * @param pSw the fixed value of pattern parameter Sw, or null if not bound.
   * @param pSp the fixed value of pattern parameter Sp, or null if not bound.
   * @param pR the fixed value of pattern parameter R, or null if not bound.
   * @return matches represented as a RouteSensorMatch object.
   * 
   */
  public Collection<RouteSensorMatch> getAllMatches(final Sensor pSen, final Switch pSw, final SwitchPosition pSp, final Route pR) {
    return rawGetAllMatches(new Object[]{pSen, pSw, pSp, pR});
  }
  
  /**
   * Returns an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pSen the fixed value of pattern parameter Sen, or null if not bound.
   * @param pSw the fixed value of pattern parameter Sw, or null if not bound.
   * @param pSp the fixed value of pattern parameter Sp, or null if not bound.
   * @param pR the fixed value of pattern parameter R, or null if not bound.
   * @return a match represented as a RouteSensorMatch object, or null if no match is found.
   * 
   */
  public RouteSensorMatch getOneArbitraryMatch(final Sensor pSen, final Switch pSw, final SwitchPosition pSp, final Route pR) {
    return rawGetOneArbitraryMatch(new Object[]{pSen, pSw, pSp, pR});
  }
  
  /**
   * Indicates whether the given combination of specified pattern parameters constitute a valid pattern match,
   * under any possible substitution of the unspecified parameters (if any).
   * @param pSen the fixed value of pattern parameter Sen, or null if not bound.
   * @param pSw the fixed value of pattern parameter Sw, or null if not bound.
   * @param pSp the fixed value of pattern parameter Sp, or null if not bound.
   * @param pR the fixed value of pattern parameter R, or null if not bound.
   * @return true if the input is a valid (partial) match of the pattern.
   * 
   */
  public boolean hasMatch(final Sensor pSen, final Switch pSw, final SwitchPosition pSp, final Route pR) {
    return rawHasMatch(new Object[]{pSen, pSw, pSp, pR});
  }
  
  /**
   * Returns the number of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pSen the fixed value of pattern parameter Sen, or null if not bound.
   * @param pSw the fixed value of pattern parameter Sw, or null if not bound.
   * @param pSp the fixed value of pattern parameter Sp, or null if not bound.
   * @param pR the fixed value of pattern parameter R, or null if not bound.
   * @return the number of pattern matches found.
   * 
   */
  public int countMatches(final Sensor pSen, final Switch pSw, final SwitchPosition pSp, final Route pR) {
    return rawCountMatches(new Object[]{pSen, pSw, pSp, pR});
  }
  
  /**
   * Executes the given processor on each match of the pattern that conforms to the given fixed values of some parameters.
   * @param pSen the fixed value of pattern parameter Sen, or null if not bound.
   * @param pSw the fixed value of pattern parameter Sw, or null if not bound.
   * @param pSp the fixed value of pattern parameter Sp, or null if not bound.
   * @param pR the fixed value of pattern parameter R, or null if not bound.
   * @param processor the action that will process each pattern match.
   * 
   */
  public void forEachMatch(final Sensor pSen, final Switch pSw, final SwitchPosition pSp, final Route pR, final IMatchProcessor<? super RouteSensorMatch> processor) {
    rawForEachMatch(new Object[]{pSen, pSw, pSp, pR}, processor);
  }
  
  /**
   * Executes the given processor on an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pSen the fixed value of pattern parameter Sen, or null if not bound.
   * @param pSw the fixed value of pattern parameter Sw, or null if not bound.
   * @param pSp the fixed value of pattern parameter Sp, or null if not bound.
   * @param pR the fixed value of pattern parameter R, or null if not bound.
   * @param processor the action that will process the selected match.
   * @return true if the pattern has at least one match with the given parameter values, false if the processor was not invoked
   * 
   */
  public boolean forOneArbitraryMatch(final Sensor pSen, final Switch pSw, final SwitchPosition pSp, final Route pR, final IMatchProcessor<? super RouteSensorMatch> processor) {
    return rawForOneArbitraryMatch(new Object[]{pSen, pSw, pSp, pR}, processor);
  }
  
  /**
   * Registers a new filtered delta monitor on this pattern matcher.
   * The DeltaMonitor can be used to track changes (delta) in the set of filtered pattern matches from now on, considering those matches only that conform to the given fixed values of some parameters.
   * It can also be reset to track changes from a later point in time,
   * and changes can even be acknowledged on an individual basis.
   * See {@link DeltaMonitor} for details.
   * @param fillAtStart if true, all current matches are reported as new match events; if false, the delta monitor starts empty.
   * @param pSen the fixed value of pattern parameter Sen, or null if not bound.
   * @param pSw the fixed value of pattern parameter Sw, or null if not bound.
   * @param pSp the fixed value of pattern parameter Sp, or null if not bound.
   * @param pR the fixed value of pattern parameter R, or null if not bound.
   * @return the delta monitor.
   * @deprecated use the IncQuery Databinding API (IncQueryObservables) instead.
   * 
   */
  @Deprecated
  public DeltaMonitor<RouteSensorMatch> newFilteredDeltaMonitor(final boolean fillAtStart, final Sensor pSen, final Switch pSw, final SwitchPosition pSp, final Route pR) {
    return rawNewFilteredDeltaMonitor(fillAtStart, new Object[]{pSen, pSw, pSp, pR});
  }
  
  /**
   * Returns a new (partial) Match object for the matcher.
   * This can be used e.g. to call the matcher with a partial match.
   * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
   * @param pSen the fixed value of pattern parameter Sen, or null if not bound.
   * @param pSw the fixed value of pattern parameter Sw, or null if not bound.
   * @param pSp the fixed value of pattern parameter Sp, or null if not bound.
   * @param pR the fixed value of pattern parameter R, or null if not bound.
   * @return the (partial) match object.
   * 
   */
  public RouteSensorMatch newMatch(final Sensor pSen, final Switch pSw, final SwitchPosition pSp, final Route pR) {
    return new RouteSensorMatch.Immutable(pSen, pSw, pSp, pR);
    
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sen.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Sensor> rawAccumulateAllValuesOfSen(final Object[] parameters) {
    Set<Sensor> results = new HashSet<Sensor>();
    rawAccumulateAllValues(POSITION_SEN, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sen.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Sensor> getAllValuesOfSen() {
    return rawAccumulateAllValuesOfSen(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sen.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Sensor> getAllValuesOfSen(final RouteSensorMatch partialMatch) {
    return rawAccumulateAllValuesOfSen(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sen.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Sensor> getAllValuesOfSen(final Switch pSw, final SwitchPosition pSp, final Route pR) {
    return rawAccumulateAllValuesOfSen(new Object[]{null, pSw, pSp, pR});
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sw.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Switch> rawAccumulateAllValuesOfSw(final Object[] parameters) {
    Set<Switch> results = new HashSet<Switch>();
    rawAccumulateAllValues(POSITION_SW, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sw.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Switch> getAllValuesOfSw() {
    return rawAccumulateAllValuesOfSw(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sw.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Switch> getAllValuesOfSw(final RouteSensorMatch partialMatch) {
    return rawAccumulateAllValuesOfSw(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sw.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Switch> getAllValuesOfSw(final Sensor pSen, final SwitchPosition pSp, final Route pR) {
    return rawAccumulateAllValuesOfSw(new Object[]{pSen, null, pSp, pR});
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sp.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<SwitchPosition> rawAccumulateAllValuesOfSp(final Object[] parameters) {
    Set<SwitchPosition> results = new HashSet<SwitchPosition>();
    rawAccumulateAllValues(POSITION_SP, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sp.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<SwitchPosition> getAllValuesOfSp() {
    return rawAccumulateAllValuesOfSp(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sp.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<SwitchPosition> getAllValuesOfSp(final RouteSensorMatch partialMatch) {
    return rawAccumulateAllValuesOfSp(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sp.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<SwitchPosition> getAllValuesOfSp(final Sensor pSen, final Switch pSw, final Route pR) {
    return rawAccumulateAllValuesOfSp(new Object[]{pSen, pSw, null, pR});
  }
  
  /**
   * Retrieve the set of values that occur in matches for R.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Route> rawAccumulateAllValuesOfR(final Object[] parameters) {
    Set<Route> results = new HashSet<Route>();
    rawAccumulateAllValues(POSITION_R, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for R.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Route> getAllValuesOfR() {
    return rawAccumulateAllValuesOfR(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for R.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Route> getAllValuesOfR(final RouteSensorMatch partialMatch) {
    return rawAccumulateAllValuesOfR(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for R.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Route> getAllValuesOfR(final Sensor pSen, final Switch pSw, final SwitchPosition pSp) {
    return rawAccumulateAllValuesOfR(new Object[]{pSen, pSw, pSp, null});
  }
  
  @Override
  protected RouteSensorMatch tupleToMatch(final Tuple t) {
    try {
    	return new RouteSensorMatch.Immutable((Concept.Sensor) t.get(POSITION_SEN), (Concept.Switch) t.get(POSITION_SW), (Concept.SwitchPosition) t.get(POSITION_SP), (Concept.Route) t.get(POSITION_R));
    } catch(ClassCastException e) {engine.getLogger().error("Element(s) in tuple not properly typed!",e);	//throw new IncQueryRuntimeException(e.getMessage());
    	return null;
    }
    
  }
  
  @Override
  protected RouteSensorMatch arrayToMatch(final Object[] match) {
    try {
    	return new RouteSensorMatch.Immutable((Concept.Sensor) match[POSITION_SEN], (Concept.Switch) match[POSITION_SW], (Concept.SwitchPosition) match[POSITION_SP], (Concept.Route) match[POSITION_R]);
    } catch(ClassCastException e) {engine.getLogger().error("Element(s) in array not properly typed!",e);	//throw new IncQueryRuntimeException(e.getMessage());
    	return null;
    }
    
  }
  
  @Override
  protected RouteSensorMatch arrayToMatchMutable(final Object[] match) {
    try {
    	return new RouteSensorMatch.Mutable((Concept.Sensor) match[POSITION_SEN], (Concept.Switch) match[POSITION_SW], (Concept.SwitchPosition) match[POSITION_SP], (Concept.Route) match[POSITION_R]);
    } catch(ClassCastException e) {engine.getLogger().error("Element(s) in array not properly typed!",e);	//throw new IncQueryRuntimeException(e.getMessage());
    	return null;
    }
    
  }
}
