package hu.bme.mit.train.constraintcheck.incquery;

import Concept.Route;
import Concept.Signal;
import Concept.Switch;
import Concept.SwitchPosition;
import hu.bme.mit.train.constraintcheck.incquery.util.SwitchSetQuerySpecification;
import java.util.Arrays;
import java.util.List;
import org.eclipse.incquery.runtime.api.IPatternMatch;
import org.eclipse.incquery.runtime.api.impl.BasePatternMatch;
import org.eclipse.incquery.runtime.exception.IncQueryException;

/**
 * Pattern-specific match representation of the hu.bme.mit.train.constraintcheck.incquery.switchSet pattern,
 * to be used in conjunction with {@link SwitchSetMatcher}.
 * 
 * <p>Class fields correspond to parameters of the pattern. Fields with value null are considered unassigned.
 * Each instance is a (possibly partial) substitution of pattern parameters,
 * usable to represent a match of the pattern in the result of a query,
 * or to specify the bound (fixed) input parameters when issuing a query.
 * 
 * @see SwitchSetMatcher
 * @see SwitchSetProcessor
 * 
 */
@SuppressWarnings("all")
public abstract class SwitchSetMatch extends BasePatternMatch {
  private Object fSpstate;
  
  private Switch fSw;
  
  private Signal fSig;
  
  private SwitchPosition fSp;
  
  private Object fSwstate;
  
  private Route fR;
  
  private static List<String> parameterNames = makeImmutableList("Spstate", "Sw", "Sig", "Sp", "Swstate", "R");
  
  private SwitchSetMatch(final Object pSpstate, final Switch pSw, final Signal pSig, final SwitchPosition pSp, final Object pSwstate, final Route pR) {
    this.fSpstate = pSpstate;
    this.fSw = pSw;
    this.fSig = pSig;
    this.fSp = pSp;
    this.fSwstate = pSwstate;
    this.fR = pR;
    
  }
  
  @Override
  public Object get(final String parameterName) {
    if ("Spstate".equals(parameterName)) return this.fSpstate;
    if ("Sw".equals(parameterName)) return this.fSw;
    if ("Sig".equals(parameterName)) return this.fSig;
    if ("Sp".equals(parameterName)) return this.fSp;
    if ("Swstate".equals(parameterName)) return this.fSwstate;
    if ("R".equals(parameterName)) return this.fR;
    return null;
    
  }
  
  public Object getSpstate() {
    return this.fSpstate;
    
  }
  
  public Switch getSw() {
    return this.fSw;
    
  }
  
  public Signal getSig() {
    return this.fSig;
    
  }
  
  public SwitchPosition getSp() {
    return this.fSp;
    
  }
  
  public Object getSwstate() {
    return this.fSwstate;
    
  }
  
  public Route getR() {
    return this.fR;
    
  }
  
  @Override
  public boolean set(final String parameterName, final Object newValue) {
    if (!isMutable()) throw new java.lang.UnsupportedOperationException();
    if ("Spstate".equals(parameterName) && newValue instanceof java.lang.Object) {
    	this.fSpstate = (java.lang.Object) newValue;
    	return true;
    }
    if ("Sw".equals(parameterName) ) {
    	this.fSw = (Concept.Switch) newValue;
    	return true;
    }
    if ("Sig".equals(parameterName) ) {
    	this.fSig = (Concept.Signal) newValue;
    	return true;
    }
    if ("Sp".equals(parameterName) ) {
    	this.fSp = (Concept.SwitchPosition) newValue;
    	return true;
    }
    if ("Swstate".equals(parameterName) && newValue instanceof java.lang.Object) {
    	this.fSwstate = (java.lang.Object) newValue;
    	return true;
    }
    if ("R".equals(parameterName) ) {
    	this.fR = (Concept.Route) newValue;
    	return true;
    }
    return false;
    
  }
  
  public void setSpstate(final Object pSpstate) {
    if (!isMutable()) throw new java.lang.UnsupportedOperationException();
    this.fSpstate = pSpstate;
    
  }
  
  public void setSw(final Switch pSw) {
    if (!isMutable()) throw new java.lang.UnsupportedOperationException();
    this.fSw = pSw;
    
  }
  
  public void setSig(final Signal pSig) {
    if (!isMutable()) throw new java.lang.UnsupportedOperationException();
    this.fSig = pSig;
    
  }
  
  public void setSp(final SwitchPosition pSp) {
    if (!isMutable()) throw new java.lang.UnsupportedOperationException();
    this.fSp = pSp;
    
  }
  
  public void setSwstate(final Object pSwstate) {
    if (!isMutable()) throw new java.lang.UnsupportedOperationException();
    this.fSwstate = pSwstate;
    
  }
  
  public void setR(final Route pR) {
    if (!isMutable()) throw new java.lang.UnsupportedOperationException();
    this.fR = pR;
    
  }
  
  @Override
  public String patternName() {
    return "hu.bme.mit.train.constraintcheck.incquery.switchSet";
    
  }
  
  @Override
  public List<String> parameterNames() {
    return SwitchSetMatch.parameterNames;
    
  }
  
  @Override
  public Object[] toArray() {
    return new Object[]{fSpstate, fSw, fSig, fSp, fSwstate, fR};
    
  }
  
  @Override
  public String prettyPrint() {
    StringBuilder result = new StringBuilder();
    result.append("\"Spstate\"=" + prettyPrintValue(fSpstate) + ", ");
    result.append("\"Sw\"=" + prettyPrintValue(fSw) + ", ");
    result.append("\"Sig\"=" + prettyPrintValue(fSig) + ", ");
    result.append("\"Sp\"=" + prettyPrintValue(fSp) + ", ");
    result.append("\"Swstate\"=" + prettyPrintValue(fSwstate) + ", ");
    result.append("\"R\"=" + prettyPrintValue(fR));
    return result.toString();
    
  }
  
  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((fSpstate == null) ? 0 : fSpstate.hashCode());
    result = prime * result + ((fSw == null) ? 0 : fSw.hashCode());
    result = prime * result + ((fSig == null) ? 0 : fSig.hashCode());
    result = prime * result + ((fSp == null) ? 0 : fSp.hashCode());
    result = prime * result + ((fSwstate == null) ? 0 : fSwstate.hashCode());
    result = prime * result + ((fR == null) ? 0 : fR.hashCode());
    return result;
    
  }
  
  @Override
  public boolean equals(final Object obj) {
    if (this == obj)
    	return true;
    if (!(obj instanceof SwitchSetMatch)) { // this should be infrequent
    	if (obj == null)
    		return false;
    	if (!(obj instanceof IPatternMatch))
    		return false;
    	IPatternMatch otherSig  = (IPatternMatch) obj;
    	if (!specification().equals(otherSig.specification()))
    		return false;
    	return Arrays.deepEquals(toArray(), otherSig.toArray());
    }
    SwitchSetMatch other = (SwitchSetMatch) obj;
    if (fSpstate == null) {if (other.fSpstate != null) return false;}
    else if (!fSpstate.equals(other.fSpstate)) return false;
    if (fSw == null) {if (other.fSw != null) return false;}
    else if (!fSw.equals(other.fSw)) return false;
    if (fSig == null) {if (other.fSig != null) return false;}
    else if (!fSig.equals(other.fSig)) return false;
    if (fSp == null) {if (other.fSp != null) return false;}
    else if (!fSp.equals(other.fSp)) return false;
    if (fSwstate == null) {if (other.fSwstate != null) return false;}
    else if (!fSwstate.equals(other.fSwstate)) return false;
    if (fR == null) {if (other.fR != null) return false;}
    else if (!fR.equals(other.fR)) return false;
    return true;
  }
  
  @Override
  public SwitchSetQuerySpecification specification() {
    try {
    	return SwitchSetQuerySpecification.instance();
    } catch (IncQueryException ex) {
     	// This cannot happen, as the match object can only be instantiated if the query specification exists
     	throw new IllegalStateException	(ex);
    }
    
  }
  
  @SuppressWarnings("all")
  static final class Mutable extends SwitchSetMatch {
    Mutable(final Object pSpstate, final Switch pSw, final Signal pSig, final SwitchPosition pSp, final Object pSwstate, final Route pR) {
      super(pSpstate, pSw, pSig, pSp, pSwstate, pR);
      
    }
    
    @Override
    public boolean isMutable() {
      return true;
    }
  }
  
  
  @SuppressWarnings("all")
  static final class Immutable extends SwitchSetMatch {
    Immutable(final Object pSpstate, final Switch pSw, final Signal pSig, final SwitchPosition pSp, final Object pSwstate, final Route pR) {
      super(pSpstate, pSw, pSig, pSp, pSwstate, pR);
      
    }
    
    @Override
    public boolean isMutable() {
      return false;
    }
  }
  
}
