package hu.bme.mit.train.constraintcheck.incquery;

import Concept.Route;
import Concept.Signal;
import Concept.Switch;
import Concept.SwitchPosition;
import hu.bme.mit.train.constraintcheck.incquery.SwitchSetMatch;
import hu.bme.mit.train.constraintcheck.incquery.util.SwitchSetQuerySpecification;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.incquery.runtime.api.IMatchProcessor;
import org.eclipse.incquery.runtime.api.IQuerySpecification;
import org.eclipse.incquery.runtime.api.IncQueryEngine;
import org.eclipse.incquery.runtime.api.impl.BaseMatcher;
import org.eclipse.incquery.runtime.exception.IncQueryException;
import org.eclipse.incquery.runtime.matchers.tuple.Tuple;
import org.eclipse.incquery.runtime.rete.misc.DeltaMonitor;

/**
 * Generated pattern matcher API of the hu.bme.mit.train.constraintcheck.incquery.switchSet pattern,
 * providing pattern-specific query methods.
 * 
 * <p>Use the pattern matcher on a given model via {@link #on(IncQueryEngine)},
 * e.g. in conjunction with {@link IncQueryEngine#on(Notifier)}.
 * 
 * <p>Matches of the pattern will be represented as {@link SwitchSetMatch}.
 * 
 * <p>Original source:
 * <code><pre>
 * // expected result: 20
 * 
 * pattern switchSet(Spstate, Sw, Sig, Sp, Swstate, R) = {
 * 	Route(R);  // there is a train rpute
 * 	
 * 	Signal(Sig);
 * 	Route.Route_entry(R, Sig); // that has an entry signal
 * 	
 * 	SignalStateKind(Individual3);
 * 	Signal.Signal_actualState(Sig, Individual3);
 * 	
 * 	Individual3 == SignalStateKind::SignalStateKind_GO; // SignalStateKind_GO
 * 	
 * 	SwitchPosition(Sp);
 * 	Route.Route_switchPosition(R, Sp); // and the route requires a switch
 * 
 * 	SwitchStateKind(Spstate);
 * 	SwitchPosition.SwitchPosition_switchState(Sp, Spstate); // and its position is Spstate
 * 
 * 	Switch(Sw);
 * 	SwitchPosition.SwitchPosition_switch(Sp, Sw); // and there is the switch
 * 	
 * 	SwitchStateKind(Swstate);
 * 	Switch.Switch_actualState(Sw, Swstate); // which is in Swstate
 * 	
 * 	Spstate != Swstate;
 * }
 * </pre></code>
 * 
 * @see SwitchSetMatch
 * @see SwitchSetProcessor
 * @see SwitchSetQuerySpecification
 * 
 */
@SuppressWarnings("all")
public class SwitchSetMatcher extends BaseMatcher<SwitchSetMatch> {
  /**
   * @return the singleton instance of the query specification of this pattern
   * @throws IncQueryException if the pattern definition could not be loaded
   * 
   */
  public static IQuerySpecification<SwitchSetMatcher> querySpecification() throws IncQueryException {
    return SwitchSetQuerySpecification.instance();
  }
  
  /**
   * Initializes the pattern matcher within an existing EMF-IncQuery engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing EMF-IncQuery engine in which this matcher will be created.
   * @throws IncQueryException if an error occurs during pattern matcher creation
   * 
   */
  public static SwitchSetMatcher on(final IncQueryEngine engine) throws IncQueryException {
    // check if matcher already exists
    SwitchSetMatcher matcher = engine.getExistingMatcher(querySpecification());
    if (matcher == null) {
    	matcher = new SwitchSetMatcher(engine);
    	// do not have to "put" it into engine.matchers, reportMatcherInitialized() will take care of it
    }
    return matcher;
  }
  
  private final static int POSITION_SPSTATE = 0;
  
  private final static int POSITION_SW = 1;
  
  private final static int POSITION_SIG = 2;
  
  private final static int POSITION_SP = 3;
  
  private final static int POSITION_SWSTATE = 4;
  
  private final static int POSITION_R = 5;
  
  /**
   * Initializes the pattern matcher over a given EMF model root (recommended: Resource or ResourceSet).
   * If a pattern matcher is already constructed with the same root, only a light-weight reference is returned.
   * The scope of pattern matching will be the given EMF model root and below (see FAQ for more precise definition).
   * The match set will be incrementally refreshed upon updates from this scope.
   * <p>The matcher will be created within the managed {@link IncQueryEngine} belonging to the EMF model root, so
   * multiple matchers will reuse the same engine and benefit from increased performance and reduced memory footprint.
   * @param emfRoot the root of the EMF containment hierarchy where the pattern matcher will operate. Recommended: Resource or ResourceSet.
   * @throws IncQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(IncQueryEngine)} instead, e.g. in conjunction with {@link IncQueryEngine#on(Notifier)}
   * 
   */
  @Deprecated
  public SwitchSetMatcher(final Notifier emfRoot) throws IncQueryException {
    this(IncQueryEngine.on(emfRoot));
  }
  
  /**
   * Initializes the pattern matcher within an existing EMF-IncQuery engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing EMF-IncQuery engine in which this matcher will be created.
   * @throws IncQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(IncQueryEngine)} instead
   * 
   */
  @Deprecated
  public SwitchSetMatcher(final IncQueryEngine engine) throws IncQueryException {
    super(engine, querySpecification());
  }
  
  /**
   * Returns the set of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pSpstate the fixed value of pattern parameter Spstate, or null if not bound.
   * @param pSw the fixed value of pattern parameter Sw, or null if not bound.
   * @param pSig the fixed value of pattern parameter Sig, or null if not bound.
   * @param pSp the fixed value of pattern parameter Sp, or null if not bound.
   * @param pSwstate the fixed value of pattern parameter Swstate, or null if not bound.
   * @param pR the fixed value of pattern parameter R, or null if not bound.
   * @return matches represented as a SwitchSetMatch object.
   * 
   */
  public Collection<SwitchSetMatch> getAllMatches(final Object pSpstate, final Switch pSw, final Signal pSig, final SwitchPosition pSp, final Object pSwstate, final Route pR) {
    return rawGetAllMatches(new Object[]{pSpstate, pSw, pSig, pSp, pSwstate, pR});
  }
  
  /**
   * Returns an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pSpstate the fixed value of pattern parameter Spstate, or null if not bound.
   * @param pSw the fixed value of pattern parameter Sw, or null if not bound.
   * @param pSig the fixed value of pattern parameter Sig, or null if not bound.
   * @param pSp the fixed value of pattern parameter Sp, or null if not bound.
   * @param pSwstate the fixed value of pattern parameter Swstate, or null if not bound.
   * @param pR the fixed value of pattern parameter R, or null if not bound.
   * @return a match represented as a SwitchSetMatch object, or null if no match is found.
   * 
   */
  public SwitchSetMatch getOneArbitraryMatch(final Object pSpstate, final Switch pSw, final Signal pSig, final SwitchPosition pSp, final Object pSwstate, final Route pR) {
    return rawGetOneArbitraryMatch(new Object[]{pSpstate, pSw, pSig, pSp, pSwstate, pR});
  }
  
  /**
   * Indicates whether the given combination of specified pattern parameters constitute a valid pattern match,
   * under any possible substitution of the unspecified parameters (if any).
   * @param pSpstate the fixed value of pattern parameter Spstate, or null if not bound.
   * @param pSw the fixed value of pattern parameter Sw, or null if not bound.
   * @param pSig the fixed value of pattern parameter Sig, or null if not bound.
   * @param pSp the fixed value of pattern parameter Sp, or null if not bound.
   * @param pSwstate the fixed value of pattern parameter Swstate, or null if not bound.
   * @param pR the fixed value of pattern parameter R, or null if not bound.
   * @return true if the input is a valid (partial) match of the pattern.
   * 
   */
  public boolean hasMatch(final Object pSpstate, final Switch pSw, final Signal pSig, final SwitchPosition pSp, final Object pSwstate, final Route pR) {
    return rawHasMatch(new Object[]{pSpstate, pSw, pSig, pSp, pSwstate, pR});
  }
  
  /**
   * Returns the number of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pSpstate the fixed value of pattern parameter Spstate, or null if not bound.
   * @param pSw the fixed value of pattern parameter Sw, or null if not bound.
   * @param pSig the fixed value of pattern parameter Sig, or null if not bound.
   * @param pSp the fixed value of pattern parameter Sp, or null if not bound.
   * @param pSwstate the fixed value of pattern parameter Swstate, or null if not bound.
   * @param pR the fixed value of pattern parameter R, or null if not bound.
   * @return the number of pattern matches found.
   * 
   */
  public int countMatches(final Object pSpstate, final Switch pSw, final Signal pSig, final SwitchPosition pSp, final Object pSwstate, final Route pR) {
    return rawCountMatches(new Object[]{pSpstate, pSw, pSig, pSp, pSwstate, pR});
  }
  
  /**
   * Executes the given processor on each match of the pattern that conforms to the given fixed values of some parameters.
   * @param pSpstate the fixed value of pattern parameter Spstate, or null if not bound.
   * @param pSw the fixed value of pattern parameter Sw, or null if not bound.
   * @param pSig the fixed value of pattern parameter Sig, or null if not bound.
   * @param pSp the fixed value of pattern parameter Sp, or null if not bound.
   * @param pSwstate the fixed value of pattern parameter Swstate, or null if not bound.
   * @param pR the fixed value of pattern parameter R, or null if not bound.
   * @param processor the action that will process each pattern match.
   * 
   */
  public void forEachMatch(final Object pSpstate, final Switch pSw, final Signal pSig, final SwitchPosition pSp, final Object pSwstate, final Route pR, final IMatchProcessor<? super SwitchSetMatch> processor) {
    rawForEachMatch(new Object[]{pSpstate, pSw, pSig, pSp, pSwstate, pR}, processor);
  }
  
  /**
   * Executes the given processor on an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pSpstate the fixed value of pattern parameter Spstate, or null if not bound.
   * @param pSw the fixed value of pattern parameter Sw, or null if not bound.
   * @param pSig the fixed value of pattern parameter Sig, or null if not bound.
   * @param pSp the fixed value of pattern parameter Sp, or null if not bound.
   * @param pSwstate the fixed value of pattern parameter Swstate, or null if not bound.
   * @param pR the fixed value of pattern parameter R, or null if not bound.
   * @param processor the action that will process the selected match.
   * @return true if the pattern has at least one match with the given parameter values, false if the processor was not invoked
   * 
   */
  public boolean forOneArbitraryMatch(final Object pSpstate, final Switch pSw, final Signal pSig, final SwitchPosition pSp, final Object pSwstate, final Route pR, final IMatchProcessor<? super SwitchSetMatch> processor) {
    return rawForOneArbitraryMatch(new Object[]{pSpstate, pSw, pSig, pSp, pSwstate, pR}, processor);
  }
  
  /**
   * Registers a new filtered delta monitor on this pattern matcher.
   * The DeltaMonitor can be used to track changes (delta) in the set of filtered pattern matches from now on, considering those matches only that conform to the given fixed values of some parameters.
   * It can also be reset to track changes from a later point in time,
   * and changes can even be acknowledged on an individual basis.
   * See {@link DeltaMonitor} for details.
   * @param fillAtStart if true, all current matches are reported as new match events; if false, the delta monitor starts empty.
   * @param pSpstate the fixed value of pattern parameter Spstate, or null if not bound.
   * @param pSw the fixed value of pattern parameter Sw, or null if not bound.
   * @param pSig the fixed value of pattern parameter Sig, or null if not bound.
   * @param pSp the fixed value of pattern parameter Sp, or null if not bound.
   * @param pSwstate the fixed value of pattern parameter Swstate, or null if not bound.
   * @param pR the fixed value of pattern parameter R, or null if not bound.
   * @return the delta monitor.
   * @deprecated use the IncQuery Databinding API (IncQueryObservables) instead.
   * 
   */
  @Deprecated
  public DeltaMonitor<SwitchSetMatch> newFilteredDeltaMonitor(final boolean fillAtStart, final Object pSpstate, final Switch pSw, final Signal pSig, final SwitchPosition pSp, final Object pSwstate, final Route pR) {
    return rawNewFilteredDeltaMonitor(fillAtStart, new Object[]{pSpstate, pSw, pSig, pSp, pSwstate, pR});
  }
  
  /**
   * Returns a new (partial) Match object for the matcher.
   * This can be used e.g. to call the matcher with a partial match.
   * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
   * @param pSpstate the fixed value of pattern parameter Spstate, or null if not bound.
   * @param pSw the fixed value of pattern parameter Sw, or null if not bound.
   * @param pSig the fixed value of pattern parameter Sig, or null if not bound.
   * @param pSp the fixed value of pattern parameter Sp, or null if not bound.
   * @param pSwstate the fixed value of pattern parameter Swstate, or null if not bound.
   * @param pR the fixed value of pattern parameter R, or null if not bound.
   * @return the (partial) match object.
   * 
   */
  public SwitchSetMatch newMatch(final Object pSpstate, final Switch pSw, final Signal pSig, final SwitchPosition pSp, final Object pSwstate, final Route pR) {
    return new SwitchSetMatch.Immutable(pSpstate, pSw, pSig, pSp, pSwstate, pR);
    
  }
  
  /**
   * Retrieve the set of values that occur in matches for Spstate.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Object> rawAccumulateAllValuesOfSpstate(final Object[] parameters) {
    Set<Object> results = new HashSet<Object>();
    rawAccumulateAllValues(POSITION_SPSTATE, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for Spstate.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Object> getAllValuesOfSpstate() {
    return rawAccumulateAllValuesOfSpstate(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for Spstate.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Object> getAllValuesOfSpstate(final SwitchSetMatch partialMatch) {
    return rawAccumulateAllValuesOfSpstate(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for Spstate.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Object> getAllValuesOfSpstate(final Switch pSw, final Signal pSig, final SwitchPosition pSp, final Object pSwstate, final Route pR) {
    return rawAccumulateAllValuesOfSpstate(new Object[]{null, pSw, pSig, pSp, pSwstate, pR});
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sw.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Switch> rawAccumulateAllValuesOfSw(final Object[] parameters) {
    Set<Switch> results = new HashSet<Switch>();
    rawAccumulateAllValues(POSITION_SW, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sw.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Switch> getAllValuesOfSw() {
    return rawAccumulateAllValuesOfSw(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sw.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Switch> getAllValuesOfSw(final SwitchSetMatch partialMatch) {
    return rawAccumulateAllValuesOfSw(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sw.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Switch> getAllValuesOfSw(final Object pSpstate, final Signal pSig, final SwitchPosition pSp, final Object pSwstate, final Route pR) {
    return rawAccumulateAllValuesOfSw(new Object[]{pSpstate, null, pSig, pSp, pSwstate, pR});
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sig.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Signal> rawAccumulateAllValuesOfSig(final Object[] parameters) {
    Set<Signal> results = new HashSet<Signal>();
    rawAccumulateAllValues(POSITION_SIG, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sig.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Signal> getAllValuesOfSig() {
    return rawAccumulateAllValuesOfSig(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sig.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Signal> getAllValuesOfSig(final SwitchSetMatch partialMatch) {
    return rawAccumulateAllValuesOfSig(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sig.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Signal> getAllValuesOfSig(final Object pSpstate, final Switch pSw, final SwitchPosition pSp, final Object pSwstate, final Route pR) {
    return rawAccumulateAllValuesOfSig(new Object[]{pSpstate, pSw, null, pSp, pSwstate, pR});
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sp.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<SwitchPosition> rawAccumulateAllValuesOfSp(final Object[] parameters) {
    Set<SwitchPosition> results = new HashSet<SwitchPosition>();
    rawAccumulateAllValues(POSITION_SP, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sp.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<SwitchPosition> getAllValuesOfSp() {
    return rawAccumulateAllValuesOfSp(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sp.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<SwitchPosition> getAllValuesOfSp(final SwitchSetMatch partialMatch) {
    return rawAccumulateAllValuesOfSp(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for Sp.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<SwitchPosition> getAllValuesOfSp(final Object pSpstate, final Switch pSw, final Signal pSig, final Object pSwstate, final Route pR) {
    return rawAccumulateAllValuesOfSp(new Object[]{pSpstate, pSw, pSig, null, pSwstate, pR});
  }
  
  /**
   * Retrieve the set of values that occur in matches for Swstate.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Object> rawAccumulateAllValuesOfSwstate(final Object[] parameters) {
    Set<Object> results = new HashSet<Object>();
    rawAccumulateAllValues(POSITION_SWSTATE, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for Swstate.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Object> getAllValuesOfSwstate() {
    return rawAccumulateAllValuesOfSwstate(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for Swstate.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Object> getAllValuesOfSwstate(final SwitchSetMatch partialMatch) {
    return rawAccumulateAllValuesOfSwstate(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for Swstate.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Object> getAllValuesOfSwstate(final Object pSpstate, final Switch pSw, final Signal pSig, final SwitchPosition pSp, final Route pR) {
    return rawAccumulateAllValuesOfSwstate(new Object[]{pSpstate, pSw, pSig, pSp, null, pR});
  }
  
  /**
   * Retrieve the set of values that occur in matches for R.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Route> rawAccumulateAllValuesOfR(final Object[] parameters) {
    Set<Route> results = new HashSet<Route>();
    rawAccumulateAllValues(POSITION_R, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for R.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Route> getAllValuesOfR() {
    return rawAccumulateAllValuesOfR(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for R.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Route> getAllValuesOfR(final SwitchSetMatch partialMatch) {
    return rawAccumulateAllValuesOfR(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for R.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Route> getAllValuesOfR(final Object pSpstate, final Switch pSw, final Signal pSig, final SwitchPosition pSp, final Object pSwstate) {
    return rawAccumulateAllValuesOfR(new Object[]{pSpstate, pSw, pSig, pSp, pSwstate, null});
  }
  
  @Override
  protected SwitchSetMatch tupleToMatch(final Tuple t) {
    try {
    	return new SwitchSetMatch.Immutable((java.lang.Object) t.get(POSITION_SPSTATE), (Concept.Switch) t.get(POSITION_SW), (Concept.Signal) t.get(POSITION_SIG), (Concept.SwitchPosition) t.get(POSITION_SP), (java.lang.Object) t.get(POSITION_SWSTATE), (Concept.Route) t.get(POSITION_R));
    } catch(ClassCastException e) {engine.getLogger().error("Element(s) in tuple not properly typed!",e);	//throw new IncQueryRuntimeException(e.getMessage());
    	return null;
    }
    
  }
  
  @Override
  protected SwitchSetMatch arrayToMatch(final Object[] match) {
    try {
    	return new SwitchSetMatch.Immutable((java.lang.Object) match[POSITION_SPSTATE], (Concept.Switch) match[POSITION_SW], (Concept.Signal) match[POSITION_SIG], (Concept.SwitchPosition) match[POSITION_SP], (java.lang.Object) match[POSITION_SWSTATE], (Concept.Route) match[POSITION_R]);
    } catch(ClassCastException e) {engine.getLogger().error("Element(s) in array not properly typed!",e);	//throw new IncQueryRuntimeException(e.getMessage());
    	return null;
    }
    
  }
  
  @Override
  protected SwitchSetMatch arrayToMatchMutable(final Object[] match) {
    try {
    	return new SwitchSetMatch.Mutable((java.lang.Object) match[POSITION_SPSTATE], (Concept.Switch) match[POSITION_SW], (Concept.Signal) match[POSITION_SIG], (Concept.SwitchPosition) match[POSITION_SP], (java.lang.Object) match[POSITION_SWSTATE], (Concept.Route) match[POSITION_R]);
    } catch(ClassCastException e) {engine.getLogger().error("Element(s) in array not properly typed!",e);	//throw new IncQueryRuntimeException(e.getMessage());
    	return null;
    }
    
  }
}
