package hu.bme.mit.train.constraintcheck.incquery.ocl2iq;

import Concept.SwitchPosition;
import hu.bme.mit.train.constraintcheck.incquery.ocl2iq.util.SwitchSetQuerySpecification;
import java.util.Arrays;
import java.util.List;
import org.eclipse.incquery.runtime.api.IPatternMatch;
import org.eclipse.incquery.runtime.api.impl.BasePatternMatch;
import org.eclipse.incquery.runtime.exception.IncQueryException;

/**
 * Pattern-specific match representation of the hu.bme.mit.train.constraintcheck.incquery.ocl2iq.SwitchSet pattern,
 * to be used in conjunction with {@link SwitchSetMatcher}.
 * 
 * <p>Class fields correspond to parameters of the pattern. Fields with value null are considered unassigned.
 * Each instance is a (possibly partial) substitution of pattern parameters,
 * usable to represent a match of the pattern in the result of a query,
 * or to specify the bound (fixed) input parameters when issuing a query.
 * 
 * @see SwitchSetMatcher
 * @see SwitchSetProcessor
 * 
 */
@SuppressWarnings("all")
public abstract class SwitchSetMatch extends BasePatternMatch {
  private SwitchPosition fSelf;
  
  private static List<String> parameterNames = makeImmutableList("self");
  
  private SwitchSetMatch(final SwitchPosition pSelf) {
    this.fSelf = pSelf;
    
  }
  
  @Override
  public Object get(final String parameterName) {
    if ("self".equals(parameterName)) return this.fSelf;
    return null;
    
  }
  
  public SwitchPosition getSelf() {
    return this.fSelf;
    
  }
  
  @Override
  public boolean set(final String parameterName, final Object newValue) {
    if (!isMutable()) throw new java.lang.UnsupportedOperationException();
    if ("self".equals(parameterName) ) {
    	this.fSelf = (Concept.SwitchPosition) newValue;
    	return true;
    }
    return false;
    
  }
  
  public void setSelf(final SwitchPosition pSelf) {
    if (!isMutable()) throw new java.lang.UnsupportedOperationException();
    this.fSelf = pSelf;
    
  }
  
  @Override
  public String patternName() {
    return "hu.bme.mit.train.constraintcheck.incquery.ocl2iq.SwitchSet";
    
  }
  
  @Override
  public List<String> parameterNames() {
    return SwitchSetMatch.parameterNames;
    
  }
  
  @Override
  public Object[] toArray() {
    return new Object[]{fSelf};
    
  }
  
  @Override
  public String prettyPrint() {
    StringBuilder result = new StringBuilder();
    result.append("\"self\"=" + prettyPrintValue(fSelf));
    return result.toString();
    
  }
  
  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((fSelf == null) ? 0 : fSelf.hashCode());
    return result;
    
  }
  
  @Override
  public boolean equals(final Object obj) {
    if (this == obj)
    	return true;
    if (!(obj instanceof SwitchSetMatch)) { // this should be infrequent
    	if (obj == null)
    		return false;
    	if (!(obj instanceof IPatternMatch))
    		return false;
    	IPatternMatch otherSig  = (IPatternMatch) obj;
    	if (!specification().equals(otherSig.specification()))
    		return false;
    	return Arrays.deepEquals(toArray(), otherSig.toArray());
    }
    SwitchSetMatch other = (SwitchSetMatch) obj;
    if (fSelf == null) {if (other.fSelf != null) return false;}
    else if (!fSelf.equals(other.fSelf)) return false;
    return true;
  }
  
  @Override
  public SwitchSetQuerySpecification specification() {
    try {
    	return SwitchSetQuerySpecification.instance();
    } catch (IncQueryException ex) {
     	// This cannot happen, as the match object can only be instantiated if the query specification exists
     	throw new IllegalStateException	(ex);
    }
    
  }
  
  @SuppressWarnings("all")
  static final class Mutable extends SwitchSetMatch {
    Mutable(final SwitchPosition pSelf) {
      super(pSelf);
      
    }
    
    @Override
    public boolean isMutable() {
      return true;
    }
  }
  
  
  @SuppressWarnings("all")
  static final class Immutable extends SwitchSetMatch {
    Immutable(final SwitchPosition pSelf) {
      super(pSelf);
      
    }
    
    @Override
    public boolean isMutable() {
      return false;
    }
  }
  
}
