package hu.bme.mit.train.constraintcheck.incquery.ocl2iq;

import Concept.SwitchPosition;
import hu.bme.mit.train.constraintcheck.incquery.ocl2iq.SwitchSetMatch;
import hu.bme.mit.train.constraintcheck.incquery.ocl2iq.util.SwitchSetQuerySpecification;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.incquery.runtime.api.IMatchProcessor;
import org.eclipse.incquery.runtime.api.IQuerySpecification;
import org.eclipse.incquery.runtime.api.IncQueryEngine;
import org.eclipse.incquery.runtime.api.impl.BaseMatcher;
import org.eclipse.incquery.runtime.exception.IncQueryException;
import org.eclipse.incquery.runtime.matchers.tuple.Tuple;
import org.eclipse.incquery.runtime.rete.misc.DeltaMonitor;

/**
 * Generated pattern matcher API of the hu.bme.mit.train.constraintcheck.incquery.ocl2iq.SwitchSet pattern,
 * providing pattern-specific query methods.
 * 
 * <p>Use the pattern matcher on a given model via {@link #on(IncQueryEngine)},
 * e.g. in conjunction with {@link IncQueryEngine#on(Notifier)}.
 * 
 * <p>Matches of the pattern will be represented as {@link SwitchSetMatch}.
 * 
 * <p>Original source:
 * <code><pre>
 * // expected result: 20
 * 
 * pattern SwitchSet(
 * 	self : SwitchPosition
 * ) {
 * 	SwitchPosition(switchPosition);
 * 	self == switchPosition;
 * 	SwitchPosition.SwitchPosition_route(self, route);
 * 	Route.Route_entry(route, signal);
 * 	Signal.Signal_actualState(signal, signalStateKind);
 * 	signalStateKind_0 == SignalStateKind::SignalStateKind_GO;
 * 	signalStateKind == signalStateKind_0;
 * 	SwitchPosition.SwitchPosition_switch(self, switch_0);
 * 	Switch.Switch_actualState(switch_0, switchStateKind);
 * 	SwitchPosition.SwitchPosition_switchState(self, switchStateKind_0);
 * 	switchStateKind != switchStateKind_0;
 * }
 * </pre></code>
 * 
 * @see SwitchSetMatch
 * @see SwitchSetProcessor
 * @see SwitchSetQuerySpecification
 * 
 */
@SuppressWarnings("all")
public class SwitchSetMatcher extends BaseMatcher<SwitchSetMatch> {
  /**
   * @return the singleton instance of the query specification of this pattern
   * @throws IncQueryException if the pattern definition could not be loaded
   * 
   */
  public static IQuerySpecification<SwitchSetMatcher> querySpecification() throws IncQueryException {
    return SwitchSetQuerySpecification.instance();
  }
  
  /**
   * Initializes the pattern matcher within an existing EMF-IncQuery engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing EMF-IncQuery engine in which this matcher will be created.
   * @throws IncQueryException if an error occurs during pattern matcher creation
   * 
   */
  public static SwitchSetMatcher on(final IncQueryEngine engine) throws IncQueryException {
    // check if matcher already exists
    SwitchSetMatcher matcher = engine.getExistingMatcher(querySpecification());
    if (matcher == null) {
    	matcher = new SwitchSetMatcher(engine);
    	// do not have to "put" it into engine.matchers, reportMatcherInitialized() will take care of it
    }
    return matcher;
  }
  
  private final static int POSITION_SELF = 0;
  
  /**
   * Initializes the pattern matcher over a given EMF model root (recommended: Resource or ResourceSet).
   * If a pattern matcher is already constructed with the same root, only a light-weight reference is returned.
   * The scope of pattern matching will be the given EMF model root and below (see FAQ for more precise definition).
   * The match set will be incrementally refreshed upon updates from this scope.
   * <p>The matcher will be created within the managed {@link IncQueryEngine} belonging to the EMF model root, so
   * multiple matchers will reuse the same engine and benefit from increased performance and reduced memory footprint.
   * @param emfRoot the root of the EMF containment hierarchy where the pattern matcher will operate. Recommended: Resource or ResourceSet.
   * @throws IncQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(IncQueryEngine)} instead, e.g. in conjunction with {@link IncQueryEngine#on(Notifier)}
   * 
   */
  @Deprecated
  public SwitchSetMatcher(final Notifier emfRoot) throws IncQueryException {
    this(IncQueryEngine.on(emfRoot));
  }
  
  /**
   * Initializes the pattern matcher within an existing EMF-IncQuery engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing EMF-IncQuery engine in which this matcher will be created.
   * @throws IncQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(IncQueryEngine)} instead
   * 
   */
  @Deprecated
  public SwitchSetMatcher(final IncQueryEngine engine) throws IncQueryException {
    super(engine, querySpecification());
  }
  
  /**
   * Returns the set of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @return matches represented as a SwitchSetMatch object.
   * 
   */
  public Collection<SwitchSetMatch> getAllMatches(final SwitchPosition pSelf) {
    return rawGetAllMatches(new Object[]{pSelf});
  }
  
  /**
   * Returns an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @return a match represented as a SwitchSetMatch object, or null if no match is found.
   * 
   */
  public SwitchSetMatch getOneArbitraryMatch(final SwitchPosition pSelf) {
    return rawGetOneArbitraryMatch(new Object[]{pSelf});
  }
  
  /**
   * Indicates whether the given combination of specified pattern parameters constitute a valid pattern match,
   * under any possible substitution of the unspecified parameters (if any).
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @return true if the input is a valid (partial) match of the pattern.
   * 
   */
  public boolean hasMatch(final SwitchPosition pSelf) {
    return rawHasMatch(new Object[]{pSelf});
  }
  
  /**
   * Returns the number of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @return the number of pattern matches found.
   * 
   */
  public int countMatches(final SwitchPosition pSelf) {
    return rawCountMatches(new Object[]{pSelf});
  }
  
  /**
   * Executes the given processor on each match of the pattern that conforms to the given fixed values of some parameters.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param processor the action that will process each pattern match.
   * 
   */
  public void forEachMatch(final SwitchPosition pSelf, final IMatchProcessor<? super SwitchSetMatch> processor) {
    rawForEachMatch(new Object[]{pSelf}, processor);
  }
  
  /**
   * Executes the given processor on an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param processor the action that will process the selected match.
   * @return true if the pattern has at least one match with the given parameter values, false if the processor was not invoked
   * 
   */
  public boolean forOneArbitraryMatch(final SwitchPosition pSelf, final IMatchProcessor<? super SwitchSetMatch> processor) {
    return rawForOneArbitraryMatch(new Object[]{pSelf}, processor);
  }
  
  /**
   * Registers a new filtered delta monitor on this pattern matcher.
   * The DeltaMonitor can be used to track changes (delta) in the set of filtered pattern matches from now on, considering those matches only that conform to the given fixed values of some parameters.
   * It can also be reset to track changes from a later point in time,
   * and changes can even be acknowledged on an individual basis.
   * See {@link DeltaMonitor} for details.
   * @param fillAtStart if true, all current matches are reported as new match events; if false, the delta monitor starts empty.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @return the delta monitor.
   * @deprecated use the IncQuery Databinding API (IncQueryObservables) instead.
   * 
   */
  @Deprecated
  public DeltaMonitor<SwitchSetMatch> newFilteredDeltaMonitor(final boolean fillAtStart, final SwitchPosition pSelf) {
    return rawNewFilteredDeltaMonitor(fillAtStart, new Object[]{pSelf});
  }
  
  /**
   * Returns a new (partial) Match object for the matcher.
   * This can be used e.g. to call the matcher with a partial match.
   * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @return the (partial) match object.
   * 
   */
  public SwitchSetMatch newMatch(final SwitchPosition pSelf) {
    return new SwitchSetMatch.Immutable(pSelf);
    
  }
  
  /**
   * Retrieve the set of values that occur in matches for self.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<SwitchPosition> rawAccumulateAllValuesOfself(final Object[] parameters) {
    Set<SwitchPosition> results = new HashSet<SwitchPosition>();
    rawAccumulateAllValues(POSITION_SELF, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for self.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<SwitchPosition> getAllValuesOfself() {
    return rawAccumulateAllValuesOfself(emptyArray());
  }
  
  @Override
  protected SwitchSetMatch tupleToMatch(final Tuple t) {
    try {
    	return new SwitchSetMatch.Immutable((Concept.SwitchPosition) t.get(POSITION_SELF));
    } catch(ClassCastException e) {engine.getLogger().error("Element(s) in tuple not properly typed!",e);	//throw new IncQueryRuntimeException(e.getMessage());
    	return null;
    }
    
  }
  
  @Override
  protected SwitchSetMatch arrayToMatch(final Object[] match) {
    try {
    	return new SwitchSetMatch.Immutable((Concept.SwitchPosition) match[POSITION_SELF]);
    } catch(ClassCastException e) {engine.getLogger().error("Element(s) in array not properly typed!",e);	//throw new IncQueryRuntimeException(e.getMessage());
    	return null;
    }
    
  }
  
  @Override
  protected SwitchSetMatch arrayToMatchMutable(final Object[] match) {
    try {
    	return new SwitchSetMatch.Mutable((Concept.SwitchPosition) match[POSITION_SELF]);
    } catch(ClassCastException e) {engine.getLogger().error("Element(s) in array not properly typed!",e);	//throw new IncQueryRuntimeException(e.getMessage());
    	return null;
    }
    
  }
}
