package hu.bme.mit.train.constraintcheck.incquery.util;

import com.google.common.collect.Sets;
import hu.bme.mit.train.constraintcheck.incquery.ConnectingSensorsMatcher;
import hu.bme.mit.train.constraintcheck.incquery.util.SensorTrackelementQuerySpecification;
import hu.bme.mit.train.constraintcheck.incquery.util.TrackelementConnectedQuerySpecification;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import org.eclipse.incquery.runtime.api.IncQueryEngine;
import org.eclipse.incquery.runtime.api.impl.BaseGeneratedQuerySpecification;
import org.eclipse.incquery.runtime.context.EMFPatternMatcherContext;
import org.eclipse.incquery.runtime.exception.IncQueryException;
import org.eclipse.incquery.runtime.extensibility.IQuerySpecificationProvider;
import org.eclipse.incquery.runtime.matchers.psystem.PBody;
import org.eclipse.incquery.runtime.matchers.psystem.PParameter;
import org.eclipse.incquery.runtime.matchers.psystem.PQuery;
import org.eclipse.incquery.runtime.matchers.psystem.PVariable;
import org.eclipse.incquery.runtime.matchers.psystem.basicdeferred.ExportedParameter;
import org.eclipse.incquery.runtime.matchers.psystem.basicenumerables.PositivePatternCall;
import org.eclipse.incquery.runtime.matchers.tuple.FlatTuple;

/**
 * A pattern-specific query specification that can instantiate ConnectingSensorsMatcher in a type-safe way.
 * 
 * @see ConnectingSensorsMatcher
 * @see ConnectingSensorsMatch
 * 
 */
@SuppressWarnings("all")
public final class ConnectingSensorsQuerySpecification extends BaseGeneratedQuerySpecification<ConnectingSensorsMatcher> {
  /**
   * @return the singleton instance of the query specification
   * @throws IncQueryException if the pattern definition could not be loaded
   * 
   */
  public static ConnectingSensorsQuerySpecification instance() throws IncQueryException {
    try {
    	return LazyHolder.INSTANCE;
    } catch (ExceptionInInitializerError err) {
    	processInitializerError(err);
    	throw err;
    }
    
  }
  
  @Override
  protected ConnectingSensorsMatcher instantiate(final IncQueryEngine engine) throws IncQueryException {
    return ConnectingSensorsMatcher.on(engine);
  }
  
  @Override
  public String getFullyQualifiedName() {
    return "hu.bme.mit.train.constraintcheck.incquery.connectingSensors";
    
  }
  
  @Override
  public List<String> getParameterNames() {
    return Arrays.asList("Sen1","Sen2");
  }
  
  @Override
  public List<PParameter> getParameters() {
    return Arrays.asList(new PParameter("Sen1", "Concept.Sensor"),new PParameter("Sen2", "Concept.Sensor"));
  }
  
  @Override
  public Set<PBody> doGetContainedBodies() throws IncQueryException {
    EMFPatternMatcherContext context = new EMFPatternMatcherContext();
    Set<PBody> bodies = Sets.newHashSet();
    {
      PBody body = new PBody(this);
      PVariable var_Sen1 = body.getOrCreateVariableByName("Sen1");
      PVariable var_Sen2 = body.getOrCreateVariableByName("Sen2");
      PVariable var_Te1 = body.getOrCreateVariableByName("Te1");
      PVariable var_Te2 = body.getOrCreateVariableByName("Te2");
      body.setExportedParameters(Arrays.asList(
        new ExportedParameter(body, var_Sen1, "Sen1"), 
        new ExportedParameter(body, var_Sen2, "Sen2")
      ));
      
      
      new PositivePatternCall(body, new FlatTuple(var_Sen1, var_Te1), SensorTrackelementQuerySpecification.instance());
      new PositivePatternCall(body, new FlatTuple(var_Sen2, var_Te2), SensorTrackelementQuerySpecification.instance());
      new PositivePatternCall(body, new FlatTuple(var_Te1, var_Te2), TrackelementConnectedQuerySpecification.instance());
      bodies.add(body);
    }setStatus(PQuery.PQueryStatus.OK);
    return bodies;
  }
  
  private ConnectingSensorsQuerySpecification() throws IncQueryException {
    super();
    setStatus(PQuery.PQueryStatus.UNINITIALIZED);
  }
  
  @SuppressWarnings("all")
  public static class Provider implements IQuerySpecificationProvider<ConnectingSensorsQuerySpecification> {
    @Override
    public ConnectingSensorsQuerySpecification get() throws IncQueryException {
      return instance();
    }
  }
  
  
  @SuppressWarnings("all")
  private static class LazyHolder {
    private final static ConnectingSensorsQuerySpecification INSTANCE = make();
    
    public static ConnectingSensorsQuerySpecification make() {
      try {
      	return new ConnectingSensorsQuerySpecification();
      } catch (IncQueryException ex) {
      	throw new RuntimeException	(ex);
      }
      
    }
  }
  
}
