package hu.bme.mit.train.constraintcheck.incquery.util;

import com.google.common.collect.Sets;
import hu.bme.mit.train.constraintcheck.incquery.HeadMatcher;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import org.eclipse.incquery.runtime.api.IncQueryEngine;
import org.eclipse.incquery.runtime.api.impl.BaseGeneratedQuerySpecification;
import org.eclipse.incquery.runtime.context.EMFPatternMatcherContext;
import org.eclipse.incquery.runtime.exception.IncQueryException;
import org.eclipse.incquery.runtime.extensibility.IQuerySpecificationProvider;
import org.eclipse.incquery.runtime.matchers.psystem.PBody;
import org.eclipse.incquery.runtime.matchers.psystem.PParameter;
import org.eclipse.incquery.runtime.matchers.psystem.PQuery;
import org.eclipse.incquery.runtime.matchers.psystem.PVariable;
import org.eclipse.incquery.runtime.matchers.psystem.basicdeferred.ExportedParameter;
import org.eclipse.incquery.runtime.matchers.psystem.basicenumerables.TypeBinary;

/**
 * A pattern-specific query specification that can instantiate HeadMatcher in a type-safe way.
 * 
 * @see HeadMatcher
 * @see HeadMatch
 * 
 */
@SuppressWarnings("all")
public final class HeadQuerySpecification extends BaseGeneratedQuerySpecification<HeadMatcher> {
  /**
   * @return the singleton instance of the query specification
   * @throws IncQueryException if the pattern definition could not be loaded
   * 
   */
  public static HeadQuerySpecification instance() throws IncQueryException {
    try {
    	return LazyHolder.INSTANCE;
    } catch (ExceptionInInitializerError err) {
    	processInitializerError(err);
    	throw err;
    }
    
  }
  
  @Override
  protected HeadMatcher instantiate(final IncQueryEngine engine) throws IncQueryException {
    return HeadMatcher.on(engine);
  }
  
  @Override
  public String getFullyQualifiedName() {
    return "hu.bme.mit.train.constraintcheck.incquery.head";
    
  }
  
  @Override
  public List<String> getParameterNames() {
    return Arrays.asList("Sen","R");
  }
  
  @Override
  public List<PParameter> getParameters() {
    return Arrays.asList(new PParameter("Sen", "Concept.Sensor"),new PParameter("R", "Concept.Route"));
  }
  
  @Override
  public Set<PBody> doGetContainedBodies() throws IncQueryException {
    EMFPatternMatcherContext context = new EMFPatternMatcherContext();
    Set<PBody> bodies = Sets.newHashSet();
    {
      PBody body = new PBody(this);
      PVariable var_Sen = body.getOrCreateVariableByName("Sen");
      PVariable var_R = body.getOrCreateVariableByName("R");
      body.setExportedParameters(Arrays.asList(
        new ExportedParameter(body, var_Sen, "Sen"), 
        new ExportedParameter(body, var_R, "R")
      ));
      
      
      new TypeBinary(body, context, var_R, var_Sen, getFeatureLiteral("http://www.semanticweb.org/ontologies/2011/1/TrainRequirementOntology.owl", "Route", "Route_routeDefinition"), "http://www.semanticweb.org/ontologies/2011/1/TrainRequirementOntology.owl/Route.Route_routeDefinition");
      bodies.add(body);
    }setStatus(PQuery.PQueryStatus.OK);
    return bodies;
  }
  
  private HeadQuerySpecification() throws IncQueryException {
    super();
    setStatus(PQuery.PQueryStatus.UNINITIALIZED);
  }
  
  @SuppressWarnings("all")
  public static class Provider implements IQuerySpecificationProvider<HeadQuerySpecification> {
    @Override
    public HeadQuerySpecification get() throws IncQueryException {
      return instance();
    }
  }
  
  
  @SuppressWarnings("all")
  private static class LazyHolder {
    private final static HeadQuerySpecification INSTANCE = make();
    
    public static HeadQuerySpecification make() {
      try {
      	return new HeadQuerySpecification();
      } catch (IncQueryException ex) {
      	throw new RuntimeException	(ex);
      }
      
    }
  }
  
}
