package hu.bme.mit.train.cc.incQuery;

import hu.bme.mit.train.cc.nativ.IQDeltaMonitor;
import hu.bme.mit.train.cc.nativ.IncQueryCommon;
import hu.bme.mit.train.constraintcheck.incquery.PosLengthMatcher;
import hu.bme.mit.train.util.BenchmarkConfig;
import hu.bme.mit.train.util.BenckmarkResult;
import hu.bme.mit.train.util.CcBenchmark;
import hu.bme.mit.train.util.EMFModification;
import hu.bme.mit.train.util.FileBroker2;
import hu.bme.mit.train.util.Util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.incquery.runtime.api.AdvancedIncQueryEngine;
import org.eclipse.incquery.runtime.exception.IncQueryException;

import Concept.IndividualContainer;
import Concept.Segment;

public class TestCasePosLength_IncQuery implements CcBenchmark {

	public static Collection<Object[]> data() {
		Object[][] data = new Object[][] { 
				{1}, {2}, {4}, {8}, {16}, {32}, {64}, {128}, {256}, {512}
		};
		return Arrays.asList(data);
	}
	
	private String fileName;
	private BenckmarkResult bmr;
	private BenchmarkConfig bc;
	private List<Segment> invalids;
	private IQDeltaMonitor<Segment> iqDeltaMonitor;
	private IndividualContainer pack;
	private ResourceSet resourceSet;
	private Resource resource;
	private AdvancedIncQueryEngine engine;

	@Override
	public void init(BenchmarkConfig bc) {
		bmr = new BenckmarkResult(this.getClass().getSimpleName());
		bmr.setSize(bc.getModelSize());
		this.fileName = "testBig" + bc.getModelVariant() + bc.getModelSize() + ".concept";
		bmr.setFileName(this.fileName);
		
		this.bc = bc;
		Util.runGC();
		if (bc.isBenchmarkMode()) {
			Util.freeCache(bc);
		}
	}
	@Override
	public void loadModel() {
		bmr.startStopper();
		
		URI resourceURI = FileBroker2.getEMFUri(bc.getInstanceModelPath() + "/" + fileName);
		resourceSet = new ResourceSetImpl();
		
		try {
			IncQueryCommon.setEIQOptions(bc);
			engine = AdvancedIncQueryEngine.createUnmanagedEngine(resourceSet);
			
			PosLengthMatcher patternMatcher = PosLengthMatcher.on(engine);
			//engine.addMatchUpdateListener(patternMatcher, new IMatchUpdateListener<PosLengthMatch>() {
			//	public void notifyAppearance(PosLengthMatch match) {};
			//	public void notifyDisappearance(PosLengthMatch match) {};
			//}, true);

			iqDeltaMonitor = new IQDeltaMonitor<Segment>(patternMatcher, "Source");
		} catch (IncQueryException e) {
			e.printStackTrace();
		}
		resource = resourceSet.getResource(resourceURI, true);
		if (resource.getContents().size() > 0 && resource.getContents().get(0) instanceof IndividualContainer) {
			pack = (IndividualContainer) resource.getContents().get(0);
		}
		
		bmr.setReadTime();
	}
	@Override
	public void check() {
		bmr.startStopper();
		
		// invalids = new ArrayList<Segment>(patternMatcher.getAllValuesOfSource());
		
		invalids = new ArrayList<Segment>(iqDeltaMonitor.getMatching());
		
		
		bmr.addInvalid(invalids.size());
		bmr.addCheckTime();
	}
	@Override
	public void modifyUserScenario() {
		EMFModification.modifyEMFmodelPosLength(pack, bmr, Util.calcModify(bc, bc.getModifyAmountPosLength(), bmr));
	}
	@Override
	public void modifyModelXFormScenario() {
		EMFModification.modifyEMFmodelPosLengthRepair(pack, bmr, Util.calcModify(bc, bc.getModifyAmountPosLength(), bmr), invalids);
	}
	@Override
	public void measureMemory() {
		Util.runGC();
		bmr.addMemoryBytes(Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory());
	}
	@Override
	public void destroy() {
		if (! engine.isManaged()) {
			engine.dispose();
		}
		engine = null;
		resource = null;
		resourceSet = null;
		Util.runGC();
	}
	@Override
	public BenckmarkResult getBMR() {
		return bmr;
	}

}
