package hu.bme.mit.train.cc.java;

import hu.bme.mit.train.util.BenchmarkConfig;
import hu.bme.mit.train.util.BenckmarkResult;
import hu.bme.mit.train.util.CcBenchmark;
import hu.bme.mit.train.util.EMFModification;
import hu.bme.mit.train.util.FileBroker;
import hu.bme.mit.train.util.FileBroker2;
import hu.bme.mit.train.util.Util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import Concept.IndividualContainer;
import Concept.Sensor;
import Concept.Switch;
import Concept.SwitchPosition;
import Concept.Trackelement;

public class TestCaseRouteSensor_Java implements CcBenchmark {

	public static Collection<Object[]> data() {
		Object[][] data = new Object[][] { 
				{1}, {2}, {4}, {8}, {16}, {32}, {64}, {128}, {256}, {512}
		};
		return Arrays.asList(data);
	}
	
	private String fileName;
	private BenckmarkResult bmr;
	private BenchmarkConfig bc;
	private List<Sensor> invalids;
	
	private IndividualContainer pack;

	private int doCCheck() {
        
        invalids = new ArrayList<Sensor>();

        for (Object eObject : pack.getContains()) {
        	if (eObject instanceof Sensor) {
        		Sensor sensor = (Sensor) eObject;
        		for(Trackelement te : sensor.getSensor_trackElement()) {
            		if (te instanceof Switch) {
            			Switch aSwitch = (Switch) te;
            			for(SwitchPosition sp : aSwitch.getSwitch_switchPosition()) {
            				if (! sp.getSwitchPosition_route().getRoute_routeDefinition().contains(sensor)) {
            					invalids.add(sensor);
            				}
            			}
            		}
        		}
        	}
        }

        /*for (Sensor sensor : invalids) {
            System.out.println(sensor);
        }*/
        
        return invalids.size();
}
	@Override
	public void init(BenchmarkConfig bc) {
		bmr = new BenckmarkResult(this.getClass().getSimpleName());
		bmr.setSize(bc.getModelSize());
		this.fileName = "models/testBig" + bc.getModelVariant() + bc.getModelSize() + ".concept";
		bmr.setFileName(this.fileName);
		
		this.bc = bc;
		Util.runGC();
		if (bc.isBenchmarkMode()) {
			Util.freeCache(bc);
		}
	}
	@Override
	public void loadModel() {
		bmr.startStopper();
		URI resourceURI = FileBroker2.getEMFUri(bc.getInstanceModelPath() + "/" + fileName);
		ResourceSet resourceSet = new ResourceSetImpl();
		Resource resource = resourceSet.getResource(resourceURI, true);

		if (resource.getContents().size() > 0 && resource.getContents().get(0) instanceof IndividualContainer) {
			pack = (IndividualContainer) resource.getContents().get(0);
		}
		bmr.setReadTime();
	}
	@Override
	public void check() {
		bmr.startStopper();
		bmr.addInvalid(doCCheck());
		bmr.addCheckTime();
	}
	@Override
	public void modifyUserScenario() {
		EMFModification.modifyEMFmodelRouteSensor(pack, bmr, Util.calcModify(bc, bc.getModifyAmountRouteSensor(), bmr));
	}
	@Override
	public void modifyModelXFormScenario() {
		EMFModification.modifyEMFmodelRouteSensorRepair(pack, bmr, Util.calcModify(bc, bc.getModifyAmountRouteSensor(), bmr), invalids);
	}
	@Override
	public void measureMemory() {
		Util.runGC();
		bmr.addMemoryBytes(Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory());
	}
	@Override
	public void destroy() {
	}
	@Override
	public BenckmarkResult getBMR() {
		return bmr;
	}

}
