package hu.bme.mit.train.cc.java;

import hu.bme.mit.train.util.BenchmarkConfig;
import hu.bme.mit.train.util.BenckmarkResult;
import hu.bme.mit.train.util.CcBenchmark;
import hu.bme.mit.train.util.EMFModification;
import hu.bme.mit.train.util.FileBroker;
import hu.bme.mit.train.util.FileBroker2;
import hu.bme.mit.train.util.Util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import Concept.IndividualContainer;
import Concept.Route;
import Concept.Sensor;
import Concept.Signal;
import Concept.Trackelement;

public class TestCaseSignalNeighbor_Java implements CcBenchmark {

	public static Collection<Object[]> data() {
		Object[][] data = new Object[][] { 
				{1} , {2}, {4}, {8}, {16} //, {32}, {64}, {128}, {256}, {512}
		};
		return Arrays.asList(data);
	}
	
	private String fileName;
	private BenckmarkResult bmr;
	private BenchmarkConfig bc;
	
	private IndividualContainer pack;
	private List<Route> matchedRoutes;

	static boolean firstCheck = true;
	private int doCCheck() {
        
        matchedRoutes = new ArrayList<Route>();
        for (Object eObject : pack.getContains()) {
        	if (eObject instanceof Route) {
        		Route aRoute = (Route) eObject;
        		if (!(isValid(aRoute))) {
        			matchedRoutes.add(aRoute);
        		}
        	}
        }

        /*
        for (Route route : invalids) {
            System.out.println(route);
        }
        */
        
        
        return matchedRoutes.size();
	}

	private boolean isValid(Route route) {
		Signal exitSignal = route.getRoute_exit();
		for (Sensor sen1 : route.getRoute_routeDefinition()) {
			for (Trackelement te1 : sen1.getSensor_trackElement()) {
				for(Trackelement te2 : te1.getTrackElement_connectsTo()) {
					for(Sensor sen2 : te2.getTrackElement_sensor()) {
						boolean goodSensor = false;
						for(Object eObject : pack.getContains()) {
							if (eObject instanceof Route) {
								Route route2X = (Route) eObject;
								if ((route2X.getRoute_routeDefinition().contains(sen2)) && (route2X != route)) {
									goodSensor = true;
									break;
								}
							}
						}
						if (goodSensor) {
							for(Object eObject : pack.getContains()) {
								if (eObject instanceof Route) {
									Route route2 = (Route) eObject;
									if ((route2.getRoute_routeDefinition().contains(sen2)) &&
											(route2.getRoute_entry() != null) && 
											(route2.getRoute_entry().equals(exitSignal))) {
										return true;
									}
								}
							}
							if (exitSignal != null) {
								return false;
							}
						}
					}
				}
			}
		}
		
		return true;
	}
	@Override
	public void init(BenchmarkConfig bc) {
		bmr = new BenckmarkResult(this.getClass().getSimpleName());
		bmr.setSize(bc.getModelSize());
		this.fileName = "models/testBig" + bc.getModelVariant() + bc.getModelSize() + ".concept";
		bmr.setFileName(this.fileName);
		
		this.bc = bc;
		Util.runGC();
		if (bc.isBenchmarkMode()) {
			Util.freeCache(bc);
		}
	}
	@Override
	public void loadModel() {
		bmr.startStopper();
		URI resourceURI = FileBroker2.getEMFUri(bc.getInstanceModelPath() + "/" + fileName);
		ResourceSet resourceSet = new ResourceSetImpl();
		Resource resource = resourceSet.getResource(resourceURI, true);

		if (resource.getContents().size() > 0 && resource.getContents().get(0) instanceof IndividualContainer) {
			pack = (IndividualContainer) resource.getContents().get(0);
		}
		bmr.setReadTime();
	}
	@Override
	public void check() {
		bmr.startStopper();
		if (firstCheck) {	
			bmr.addInvalid(doCCheck());
			if (bc.getScenarioVariant() >= 1) firstCheck = false;
		}
		bmr.addCheckTime();
	}
	@Override
	public void modifyUserScenario() {
		if (bc.getScenarioVariant() < 1)
			EMFModification.modifyEMFmodelSignalNeighbor(pack, bmr, Util.calcModify(bc, bc.getModifyAmountSignalNeighbor(), bmr));
	}
	@Override
	public void modifyModelXFormScenario() {
		if (bc.getScenarioVariant() < 1)
			EMFModification.modifyEMFmodelSignalNeighborRepair(pack, bmr, Util.calcModify(bc, bc.getModifyAmountSignalNeighbor(), bmr), matchedRoutes);
	}
	@Override
	public void measureMemory() {
		Util.runGC();
		bmr.addMemoryBytes(Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory());
	}
	@Override
	public void destroy() {
	}
	@Override
	public BenckmarkResult getBMR() {
		return bmr;
	}

}
