package hu.bme.mit.train.cc.java;

import hu.bme.mit.train.util.BenchmarkConfig;
import hu.bme.mit.train.util.BenckmarkResult;
import hu.bme.mit.train.util.CcBenchmark;
import hu.bme.mit.train.util.EMFModification;
import hu.bme.mit.train.util.FileBroker;
import hu.bme.mit.train.util.Util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import Concept.IndividualContainer;
import Concept.Route;
import Concept.Signal;
import Concept.SignalStateKind;
import Concept.SwitchPosition;

public class TestCaseSwitchSet_Java implements CcBenchmark {

	public static Collection<Object[]> data() {
		Object[][] data = new Object[][] { 
				{1}, {2}, {4}, {8}, {16}, {32}, {64}, {128}, {256}, {512}
		};
		return Arrays.asList(data);
	}
	
	private String fileName;
	private BenckmarkResult bmr;
	private BenchmarkConfig bc;
	
	private IndividualContainer pack;
	private List<SwitchPosition> invalids;

	private int doCCheck() {
        
        invalids = new ArrayList<SwitchPosition>();
        
        for (Object eObject : pack.getContains()) {
        	if (eObject instanceof SwitchPosition) {
        		SwitchPosition sp = (SwitchPosition) eObject;
        		Route route = sp.getSwitchPosition_route();
        		if (route != null) {
        			Signal entrySignal = route.getRoute_entry();
        			if (entrySignal != null) {
        				if(entrySignal.getSignal_actualState() == SignalStateKind.SIGNAL_STATE_KIND_GO) {
        					if (sp.getSwitchPosition_switch().getSwitch_actualState() != sp.getSwitchPosition_switchState()) {
        						invalids.add(sp);
        					}
        				}
        			}
        		}
        	}
        }

        /*for (SwitchPosition switchPosition : invalids) {
            System.out.println(switchPosition);
        }*/
        
        return invalids.size();
}
	@Override
	public void init(BenchmarkConfig bc) {
		bmr = new BenckmarkResult(this.getClass().getSimpleName());
		bmr.setSize(bc.getModelSize());
		this.fileName = "models/testBig" + bc.getModelVariant() + bc.getModelSize() + ".concept";
		bmr.setFileName(this.fileName);
		
		this.bc = bc;
		Util.runGC();
		if (bc.isBenchmarkMode()) {
			Util.freeCache(bc);
		}
	}
	@Override
	public void loadModel() {
		bmr.startStopper();
		URI resourceURI = FileBroker.getBigTestModelFile(fileName);
		ResourceSet resourceSet = new ResourceSetImpl();
		Resource resource = resourceSet.getResource(resourceURI, true);

		if (resource.getContents().size() > 0 && resource.getContents().get(0) instanceof IndividualContainer) {
			pack = (IndividualContainer) resource.getContents().get(0);
		}
		bmr.setReadTime();
	}
	@Override
	public void check() {
		bmr.startStopper();
		bmr.addInvalid(doCCheck());
		bmr.addCheckTime();
	}
	@Override
	public void modifyUserScenario() {
		EMFModification.modifyEMFmodelSwitchSet(pack, bmr, Util.calcModify(bc, bc.getModifyAmountSwitchSet(), bmr));
	}
	@Override
	public void modifyModelXFormScenario() {
		// throw new RuntimeException(new OperationNotSupportedException());
	}
	@Override
	public void measureMemory() {
		Util.runGC();
		bmr.addMemoryBytes(Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory());
	}
	@Override
	public void destroy() {
	}
	@Override
	public BenckmarkResult getBMR() {
		return bmr;
	}

}
