package hu.bme.mit.train.cc.mdtoclai;

import hu.bme.mit.train.common.RuntimeConstants;
import hu.bme.mit.train.util.BenchmarkConfig;
import hu.bme.mit.train.util.BenckmarkResult;
import hu.bme.mit.train.util.CcBenchmark;
import hu.bme.mit.train.util.EMFModification;
import hu.bme.mit.train.util.FileBroker;
import hu.bme.mit.train.util.Util;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.ocl.ecore.OCL;
import org.eclipse.ocl.ecore.OCLExpression;
import org.eclipse.ocl.ecore.OCL.Helper;

import Concept.ConceptPackage;
import Concept.IndividualContainer;
import Concept.Sensor;

public class TestCaseRouteSensor_OCLAI implements CcBenchmark {

	public static Collection<Object[]> data() {
		Object[][] data = new Object[][] { 
				{1}, {2}, {4}, {8}, {16}, {32} //, {64}, {128}, {256}, {512}
		};
		return Arrays.asList(data);
	}
	
	private String fileName;
	private BenckmarkResult bmr;
	
	private BenchmarkConfig bc;
	
	private IndividualContainer pack;

	private OCL ocl;
	private OCLExpression query;
	private String oclConstraint;
	private List<Sensor> invalids;

	private int doCCheck() {
        
        invalids = new ArrayList<Sensor>();
        if (query != null) {
            for (Object eObject : pack.getContains()) {
                Object result = ocl.evaluate(eObject, query);
                if (result == Boolean.TRUE) {
                    invalids.add((Sensor) eObject);
                }
            }
        }

        /*for (Sensor sensor : invalids) {
            System.out.println(sensor);
        }*/
        
        return invalids.size();
}
	@Override
	public void init(BenchmarkConfig bc) {
		bmr = new BenckmarkResult(this.getClass().getSimpleName());
		bmr.setSize(bc.getModelSize());
		this.fileName = "models/testBig" + bc.getModelVariant() + bc.getModelSize() + ".concept";
		bmr.setFileName(this.fileName);
		
		this.bc = bc;
		Util.runGC();
		if (bc.isBenchmarkMode()) {
			Util.freeCache(bc);
		}
	}
	@Override
	public void loadModel() {
		bmr.startStopper();
		URI resourceURI = FileBroker.getBigTestModelFile(fileName);
		ResourceSet resourceSet = new ResourceSetImpl();
		Resource resource = resourceSet.getResource(resourceURI, true);

		// ************ TESTCASE related ************

		if (resource.getContents().size() > 0 && resource.getContents().get(0) instanceof IndividualContainer) {
			pack = (IndividualContainer) resource.getContents().get(0);
		}

		try {
			oclConstraint = Util.readFile(RuntimeConstants.projectPath + "/models/query/ocl/RouteSensor.oclai");
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		ocl = OCL.newInstance();
        Helper helper = ocl.createOCLHelper();
        query = null;
        try {
            helper.setContext(ConceptPackage.eINSTANCE.getSensor());
            query = helper.createQuery(oclConstraint);
        } catch (org.eclipse.ocl.ParserException e) {
            e.printStackTrace();
        }
		bmr.setReadTime();
	}
	@Override
	public void check() {
		bmr.startStopper();
		bmr.addInvalid(doCCheck());
		bmr.addCheckTime();
	}
	@Override
	public void modifyUserScenario() {
		EMFModification.modifyEMFmodelRouteSensor(pack, bmr, Util.calcModify(bc, bc.getModifyAmountRouteSensor(), bmr));
	}
	@Override
	public void modifyModelXFormScenario() {
		EMFModification.modifyEMFmodelRouteSensorRepair(pack, bmr, Util.calcModify(bc, bc.getModifyAmountRouteSensor(), bmr), invalids);
	}
	@Override
	public void measureMemory() {
		Util.runGC();
		bmr.addMemoryBytes(Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory());
	}
	@Override
	public void destroy() {
		ocl.dispose();
	}
	@Override
	public BenckmarkResult getBMR() {
		return bmr;
	}

}
