package hu.bme.mit.train.scenarios;

import java.util.Collection;

import hu.bme.mit.train.util.BenchmarkConfig;

public class GenericBenchmarkLogic {
	
	protected BenchmarkConfig bc = new BenchmarkConfig();
	private TestFactory testFactory;
	
	public GenericBenchmarkLogic(String[] args, TestFactory testFactory) {
		bc.processParameters(args);
		this.testFactory = testFactory;
	}

	public void runBenchmarks() {
		ScenarioLogic scl;
		for(String scenario : bc.getScenarios()) {
			bc.setCurrentScenario(scenario);
			if (bc.doTestCase("PosLength")) {
				Collection<Object[]> sizes;
				if(bc.getSizes() != null)
					sizes = bc.getSizes();
				else
					sizes = testFactory.getPosLengthModelsizes();
				for(Object[] modelSizes : sizes) {
					for(int sizeIdx = 0; sizeIdx < modelSizes.length; sizeIdx++) {
						int modelSize = (Integer) modelSizes[sizeIdx];
						scl = ScenarioFactory.getScenarioLogic(scenario);
						bc.setModelSize(modelSize);
						scl.runBenchmark(bc, testFactory.getPosLength());
						scl = null;
					}
				}
			} else if (bc.doTestCase("RouteSensor")) {
				Collection<Object[]> sizes;
				if(bc.getSizes() != null)
					sizes = bc.getSizes();
				else
					sizes = testFactory.getRouteSensorModelsizes();
				for(Object[] modelSizes : sizes) {
					for(int sizeIdx = 0; sizeIdx < modelSizes.length; sizeIdx++) {
						int modelSize = (Integer) modelSizes[sizeIdx];
						scl = ScenarioFactory.getScenarioLogic(scenario);
						bc.setModelSize(modelSize);
						scl.runBenchmark(bc, testFactory.getRouteSensor());
						scl = null;
					}
				}
			} else if (bc.doTestCase("SwitchSensor")) {
				Collection<Object[]> sizes;
				if(bc.getSizes() != null)
					sizes = bc.getSizes();
				else
					sizes = testFactory.getSwitchSensorModelsizes();
				for(Object[] modelSizes : sizes) {
					for(int sizeIdx = 0; sizeIdx < modelSizes.length; sizeIdx++) {
						int modelSize = (Integer) modelSizes[sizeIdx];
						scl = ScenarioFactory.getScenarioLogic(scenario);
						bc.setModelSize(modelSize);
						scl.runBenchmark(bc, testFactory.getSwitchSensor());
						scl = null;
					}
				}
			} else if (bc.doTestCase("SwitchSet")) {
				Collection<Object[]> sizes;
				if(bc.getSizes() != null)
					sizes = bc.getSizes();
				else
					sizes = testFactory.getSwitchSetModelsizes();
				for(Object[] modelSizes : sizes) {
					for(int sizeIdx = 0; sizeIdx < modelSizes.length; sizeIdx++) {
						int modelSize = (Integer) modelSizes[sizeIdx];
						scl = ScenarioFactory.getScenarioLogic(scenario);
						bc.setModelSize(modelSize);
						scl.runBenchmark(bc, testFactory.getSwitchSet());
						scl = null;
					}
				}
			} else if (bc.doTestCase("SignalNeighbor")) {
				Collection<Object[]> sizes;
				if(bc.getSizes() != null)
					sizes = bc.getSizes();
				else
					sizes = testFactory.getSignalNeighborModelsizes();
				for(Object[] modelSizes : sizes) {
					for(int sizeIdx = 0; sizeIdx < modelSizes.length; sizeIdx++) {
						int modelSize = (Integer) modelSizes[sizeIdx];
						scl = ScenarioFactory.getScenarioLogic(scenario);
						bc.setModelSize(modelSize);
						scl.runBenchmark(bc, testFactory.getSignalNeighbor());
						scl = null;
					}
				}
			} else if (bc.doTestCase("RouteSensorAI")) {
				Collection<Object[]> sizes;
				if(bc.getSizes() != null)
					sizes = bc.getSizes();
				else
					sizes = testFactory.getRouteSensorAIModelsizes();
				for(Object[] modelSizes : sizes) {
					for(int sizeIdx = 0; sizeIdx < modelSizes.length; sizeIdx++) {
						int modelSize = (Integer) modelSizes[sizeIdx];
						scl = ScenarioFactory.getScenarioLogic(scenario);
						bc.setModelSize(modelSize);
						scl.runBenchmark(bc, testFactory.getRouteSensorAI());
						scl = null;
					}
				}
			} else if (bc.doTestCase("SwitchSetAI")) {
				Collection<Object[]> sizes;
				if(bc.getSizes() != null)
					sizes = bc.getSizes();
				else
					sizes = testFactory.getSwitchSetAIModelsizes();
				for(Object[] modelSizes : sizes) {
					for(int sizeIdx = 0; sizeIdx < modelSizes.length; sizeIdx++) {
						int modelSize = (Integer) modelSizes[sizeIdx];
						scl = ScenarioFactory.getScenarioLogic(scenario);
						bc.setModelSize(modelSize);
						scl.runBenchmark(bc, testFactory.getSwitchSetAI());
						scl = null;
					}
				}
			} else {
				System.err.println("Unrecognized benchmark type! Please select a benchmkar with e.g. -SwitchSensor option.");
			}
		}
	}
}
