package hu.bme.mit.train.scenarios;

import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import hu.bme.mit.train.util.BenchmarkConfig;
import hu.bme.mit.train.util.BenckmarkResult;
import hu.bme.mit.train.util.CcBenchmark;
import hu.bme.mit.train.util.Util;

@SuppressWarnings("unused") // executor can be used instead of linux timeout
public class UserScenario implements ScenarioLogic {
	@Override
	public void runBenchmark(final BenchmarkConfig bc, final CcBenchmark benchmark) {
		final String runtimeInfoFilename = benchmark.getClass().getName() + "_" + bc.getCurrentScenario();
		if (bc.isCreateSandbox()) {
//			// sandbox with Executor Service <- not working
//			try {
//				if (! Util.isFailedPrevTest(bc, runtimeInfoFilename)) {
//					Util.startTest(bc, runtimeInfoFilename);
//					ExecutorService executor = Executors.newSingleThreadExecutor();
//					try {
//						Future<String> future = executor.submit(new Callable<String>() {
//							@Override
//							public String call() throws Exception {
//								runLogic(bc, benchmark);
//								return null;
//							}
//						});
//						future.get(bc.getTimeout(), TimeUnit.MILLISECONDS);
//						executor.shutdownNow();
//						Util.doneTest(bc, runtimeInfoFilename);
//					} catch (Exception e) {
//						executor.shutdownNow();
//						Util.failedTest(bc, runtimeInfoFilename, e);
//						throw e;
//					} catch (Error e) {
//						executor.shutdownNow();
//						Util.failedTest(bc, runtimeInfoFilename, e);
//						throw e;
//					}
//				} else {
//					throw new Exception("The test is not run, because previous is failed.");
//				}
//			} catch (Exception e) {
//				e.printStackTrace();
//			}
			
			
			
			// create sandbox with linux timeout
			try {
				if (! Util.isFailedPrevTest(bc, runtimeInfoFilename)) {
					Util.startTest(bc, runtimeInfoFilename);
					runLogic(bc, benchmark);
					Util.doneTest(bc, runtimeInfoFilename);
				} else {
					System.err.println("The test is not run, because previous is failed.");
					//throw new Exception("The test is not run, because previous is failed.");
				}
			} catch (Exception e) {
				Util.failedTest(bc, runtimeInfoFilename, e);
				e.printStackTrace();
			} catch (Error e) {
				Util.failedTest(bc, runtimeInfoFilename, e);
				e.printStackTrace();
			}
		} else {
			runLogic(bc, benchmark);
		}
	}
	
	public void runLogic(BenchmarkConfig bc, CcBenchmark benchmark) {
		bc.setModelVariant("_User_");
		
		benchmark.init(bc);
		
		benchmark.loadModel();
		benchmark.check();
		for (int i = 0; i < bc.getUserIterate(); i++) {
			benchmark.modifyUserScenario();
			benchmark.check();
		}
		benchmark.measureMemory();
		benchmark.destroy();
		
		BenckmarkResult bmr = benchmark.getBMR();
		bmr.setScenario(bc.getCurrentScenario());
		System.out.println(bmr);
	}
}
