package hu.bme.mit.train.util;

import hu.bme.mit.train.common.RuntimeConstants;

import java.util.ArrayList;
import java.util.Collection;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;

public class BenchmarkConfig {
	private ArrayList<String> testCases;
	private int modifyMethod;
	private int modifyAmount;
	private int modifyAmountPosLength;
	private int modifyAmountRouteSensor;
	private int modifyAmountSwitchSensor;
	private int modifyAmountSwitchSet;
	private int modifyAmountSignalNeighbor;
	private String modelVariant;
	private int modelSize;
	private int inferencing;
	private long timeout;
	private boolean createSandbox;
	private boolean benchmarkMode;
	private ArrayList<String> scenarios;
	private String currentScenario;
	private int userIterate;
	private int divPossiblyBad;
	private Collection<Object[]> sizes;
	public String instanceModelPath;
	public String pathProject_InstanceModels;
	public String pathWorkspace;
	private boolean neoIndex;
	private String instanceVariant;
	private String queryVariant;
	private String functionalDependencyOption;
	private String stubTrimOption;
	
	// 0: default
	// 1: batch only
	private int scenarioVariant;
	
	@Override
	public String toString() {
		return "Current scenario: " + currentScenario + "\n" +
				"modelSize: " + modelSize + "\n" +
				"modelVariant: \"" + modelVariant + "\"\n" +
				"createSandbox: " + createSandbox + "\n" +
				"benchmarkMode: " + benchmarkMode + "\n" +
				"timeout: " + timeout + "\n" +
				"inferencing: " + inferencing + "\n" +
				"modifyAmount: " + getModifyAmount() + "\n" +
				"modifyMethod: " + modifyMethod + "\n" +
				"userIterate: " + userIterate + "\n" +
				"testCases: " + testCases.toString() + "\n";
	}
	
	public BenchmarkConfig() {
		testCases = new ArrayList<String>();
		inferencing = -1;
		timeout = 600000; // 10 mins
		createSandbox = false;
		benchmarkMode = false;
		scenarios = new ArrayList<String>();
		userIterate = 10;
		modifyMethod = 0;  // default is -modify N
		modifyAmount = 10; // default N is 10
		modifyAmountPosLength = 10;
		modifyAmountRouteSensor = 10;
		modifyAmountSwitchSensor = 10;
		modifyAmountSwitchSet = 10;
		modifyAmountSignalNeighbor = 10;
	}
	
	public void processParameters(String[] args) {
		Options options = new Options();
		options.addOption("modify", true, "modify fix number of elements");
		options.addOption("divisor", true, "modify allElements/divisor in the model");
		options.addOption("factor", true, "modify allElements/minElements*factor in the model");
		options.addOption("reSet", true, "modify result set/reSet");
		options.addOption("divPossiblyBad", true, "modify result possibly bad element size/N");
		
		options.addOption("sizes", true, "test sizes, e.g. 1-8 or 1,8-512");
		options.addOption("variant", true, "modification calculation variant to run");
		options.addOption("inferencing", true, "type of inference");
		options.addOption("timeout", true, "millisecs to wait for the test to run. If createSandbox is set, beyond this time the test will be killed.");
		options.addOption("createSandbox", true, "kill the test when timeout is reached, and this is true");
		options.addOption("benchmarkMode", true, "run benchmark specific (non-functional) procedures, like cleaning the OS cache.");
		
		options.addOption("ModelXFormScenario", false, "run ModelXFormScenario scenario.");
		options.addOption("UserScenario", false, "run UserScenario scenario that simulates an editing scenario of a user.");

		options.addOption("userIterate", true, "When UserScenario is used, how many modify iteration is done.");
	
		options.addOption("PosLength", false, "run the PosLength test");
		options.addOption("RouteSensor", false, "run the RouteSensor test");
		options.addOption("SwitchSensor", false, "run the SwitchSensor test");
		options.addOption("SwitchSet", false, "run the SwitchSet test");
		options.addOption("SignalNeighbor", false, "run the SignalNeighbor test");
		options.addOption("RouteSensorAI", false, "run the RouteSensorAI test");
		options.addOption("SwitchSetAI", false, "run the SwitchSetAI test");
		
		options.addOption("modifyMethod", true, "method of modification");
		options.addOption("modifyAmountPosLength", true, "amount of modification for the PosLength test");
		options.addOption("modifyAmountRouteSensor", true, "amount of modification for the RouteSensor test");
		options.addOption("modifyAmountSwitchSensor", true, "amount of modification for the SwitchSensor test");
		options.addOption("modifyAmountSwitchSet", true, "amount of modification for the SwitchSet test");
		options.addOption("modifyAmountSignalNeighbor", true, "amount of modification for the SignalNeighbor test");
		
		options.addOption("instanceModelPath", true, "directory path where the models will be generated");
		options.addOption("pathProject_InstanceModels", true, "path of the hu.bme.mit.Train.InstanceModels project");
		options.addOption("pathWorkspace", true, "path of the Eclipse workspace with all projects");
		
		options.addOption("NeoIndex", true, "use index for Neo4j");
		options.addOption("instanceVariant", true, "append to instance model filename");
		options.addOption("queryVariant", true, "select query variant");
		options.addOption("scenarioVariant", true, "variant, see specific code");
		
		// incquery specific
		options.addOption("functionalDependencyOption", true, "EIQ: FunctionalDependencyOption");
		options.addOption("stubTrimOption", true, "EIQ: StubTrimOption");
		
		CommandLineParser parser = new PosixParser();
		CommandLine cmd;
		
		try {
			cmd = parser.parse( options, args);
			if (cmd.getOptionValue("variant") != null) {
				int variant = new Integer(cmd.getOptionValue("variant"));
				if ((variant & 1) == 1) {
					testCases.add("PosLength");
					testCases.add("RouteSensor");
					testCases.add("SwitchSensor");
					testCases.add("SwitchSet");
					testCases.add("SignalNeighbor");
				}
				if ((variant & 2) == 2) {
					testCases.add("RouteSensorAI");
					testCases.add("SwitchSetAI");
				}
			}
			
			if (cmd.hasOption("PosLength"))
				testCases.add("PosLength");
			if (cmd.hasOption("RouteSensor"))
				testCases.add("RouteSensor");
			if (cmd.hasOption("SwitchSensor"))
				testCases.add("SwitchSensor");
			if (cmd.hasOption("SwitchSet"))
				testCases.add("SwitchSet");
			if (cmd.hasOption("SignalNeighbor"))
				testCases.add("SignalNeighbor");
			if (cmd.hasOption("RouteSensorAI"))
				testCases.add("RouteSensorAI");
			if (cmd.hasOption("SwitchSetAI"))
				testCases.add("SwitchSetAI");
			
			if (cmd.hasOption("ModelXFormScenario"))
				scenarios.add("ModelXFormScenario");
			if (cmd.hasOption("UserScenario"))
				scenarios.add("UserScenario");
			if (scenarios.size() == 0)
				scenarios.add("ModelXFormScenario");  // do something without parameter too
			
			if (cmd.getOptionValue("reSet") != null) {
				modifyMethod = 8;
				setModifyAmount(new Integer(cmd.getOptionValue("reSet")));
			}
			
			if (cmd.getOptionValue("modify") != null) {
				modifyMethod = 1;
				setModifyAmount(new Integer(cmd.getOptionValue("modify")));
			}
			if (cmd.getOptionValue("divisor") != null) {
				modifyMethod = 2;
				setModifyAmount(new Integer(cmd.getOptionValue("divisor")));
			}
			if (cmd.getOptionValue("divPossiblyBad") != null) {
				modifyMethod = 16;
				setModifyAmount(new Integer(cmd.getOptionValue("divPossiblyBad")));
			}
			if (cmd.getOptionValue("factor") != null) {
				modifyMethod = 4;
				setModifyAmount(new Integer(cmd.getOptionValue("factor")));
			}
			if (cmd.getOptionValue("userIterate") != null) {
				setUserIterate(new Integer(cmd.getOptionValue("userIterate")));
			}
			
			sizes = null;
			if (cmd.getOptionValue("sizes") != null) {
				sizes = new ArrayList<Object[]>();
				String sizesStr = cmd.getOptionValue("sizes");
				String[] sizeRanges = sizesStr.split(",");
				for(String sizeRange : sizeRanges) {
					Integer from = new Integer(sizeRange.split("-")[0]);
					Integer to;
					try {
						to = new Integer(sizeRange.split("-")[1]);
					} catch(ArrayIndexOutOfBoundsException e) {
						to = from; // only number without dash
					}
					if (from != 0) {
						for(int i = from; i <= to; i *= 2) {
							sizes.add(new Integer[]{i});
						}
					}
				}
			}
			if (cmd.getOptionValue("inferencing") != null) {
				setInferencing(new Integer(cmd.getOptionValue("inferencing")));
			}
			if (cmd.getOptionValue("timeout") != null) {
				setTimeout(new Long(cmd.getOptionValue("timeout")));
			}
			if (cmd.getOptionValue("createSandbox") != null) {
				setCreateSandbox(new Boolean(cmd.getOptionValue("createSandbox")));
			}
			if (cmd.getOptionValue("benchmarkMode") != null) {
				setBenchmarkMode(new Boolean(cmd.getOptionValue("benchmarkMode")));
			}
			
			
			if (cmd.getOptionValue("modifyMethod") != null) {
				modifyMethod = new Integer(cmd.getOptionValue("modifyMethod"));
			}
			if (cmd.getOptionValue("modifyAmountPosLength") != null) {
				modifyAmountPosLength = new Integer(cmd.getOptionValue("modifyAmountPosLength"));
			}
			if (cmd.getOptionValue("modifyAmountRouteSensor") != null) {
				modifyAmountRouteSensor = new Integer(cmd.getOptionValue("modifyAmountRouteSensor"));
			}
			if (cmd.getOptionValue("modifyAmountSwitchSensor") != null) {
				modifyAmountSwitchSensor = new Integer(cmd.getOptionValue("modifyAmountSwitchSensor"));
			}
			if (cmd.getOptionValue("modifyAmountSwitchSet") != null) {
				modifyAmountSwitchSet = new Integer(cmd.getOptionValue("modifyAmountSwitchSet"));
			}
			if (cmd.getOptionValue("modifyAmountSignalNeighbor") != null) {
				modifyAmountSignalNeighbor = new Integer(cmd.getOptionValue("modifyAmountSignalNeighbor"));
			}
			
			if (cmd.getOptionValue("pathWorkspace") != null) {
				pathWorkspace = cmd.getOptionValue("pathWorkspace");
			} else {
				pathWorkspace = RuntimeConstants.workspacePath;
			}
			if (cmd.getOptionValue("pathProject_InstanceModels") != null) {
				pathProject_InstanceModels = cmd.getOptionValue("pathProject_InstanceModels");
			} else {
				pathProject_InstanceModels = pathWorkspace + "/hu.bme.mit.Train.InstanceModels";
			}
			if (cmd.getOptionValue("instanceModelPath") != null) {
				instanceModelPath = cmd.getOptionValue("instanceModelPath");
			} else {
				instanceModelPath = pathProject_InstanceModels + "/models"; 
			}

			if (cmd.getOptionValue("NeoIndex") != null) {
				setNeoIndex(new Boolean(cmd.getOptionValue("NeoIndex")));
			} else {
				setNeoIndex(new Boolean(false));
			}

			if (cmd.getOptionValue("instanceVariant") != null) {
				instanceVariant = cmd.getOptionValue("instanceVariant");
				if ("-".equals(instanceVariant))
					instanceVariant = "";
			} else {
				instanceVariant = "";
			}

			if (cmd.getOptionValue("queryVariant") != null) {
				queryVariant = cmd.getOptionValue("queryVariant");
				if ("-".equals(queryVariant))
					queryVariant = "";
			} else {
				queryVariant = "";
			}

			if (cmd.getOptionValue("scenarioVariant") != null) {
				scenarioVariant = new Integer(cmd.getOptionValue("scenarioVariant"));
			} else {
				scenarioVariant = 0;
			}
			
			// EIQ specific
			if (cmd.getOptionValue("functionalDependencyOption") != null) {
				functionalDependencyOption = cmd.getOptionValue("functionalDependencyOption");
			} else {
				functionalDependencyOption = null;
			}
			if (cmd.getOptionValue("stubTrimOption") != null) {
				stubTrimOption = cmd.getOptionValue("stubTrimOption");
			} else {
				stubTrimOption = null;
			}
			
		} catch (ParseException e) {
			e.printStackTrace();
		}
	}
	

	/**
	 * 
	 * @param testCase
	 * @return true, if the testcase was specified in the arguments. If no testcase was specified, return true. (Default is to run all testcases.)
	 */
	public boolean doTestCase(String testCase) {
		if (testCases.size() > 0) 
			return testCases.contains(testCase);
		return true;
	}

	public ArrayList<String> getScenarios() {
		return scenarios;
	}
	
	public void switchDoAI() {
		if (testCases.contains("RouteSensorAI")) {
			testCases.remove("RouteSensorAI");
			testCases.remove("SwitchSetAI");
		} else {
			testCases.add("RouteSensorAI");
			testCases.add("SwitchSetAI");
		}
	}
	public boolean getDoAI() {
		return testCases.contains("RouteSensorAI");
	}

	public int getModifyMethod() {
		return modifyMethod;
	}

	public void setModifyMethod(int modifyMethod) {
		this.modifyMethod = modifyMethod;
	}

	public int getModifyAmount() {
		return modifyAmount;
	}

	public void setModifyAmount(int modifyAmount) {
				this.modifyAmount = 
				modifyAmountPosLength = 
				modifyAmountRouteSensor = 
				modifyAmountSwitchSensor =
				modifyAmountSwitchSet =
				modifyAmountSignalNeighbor =
				modifyAmount;
	}

	public int getModifyAmountPosLength() {
		return modifyAmountPosLength;
	}
	public int getModifyAmountRouteSensor() {
		return modifyAmountRouteSensor;
	}
	public int getModifyAmountSwitchSensor() {
		return modifyAmountSwitchSensor;
	}
	public int getModifyAmountSwitchSet() {
		return modifyAmountSwitchSet;
	}
	public int getModifyAmountSignalNeighbor() {
		return modifyAmountSignalNeighbor;
	}
	public int getModifyAmountRouteSensorAI() {
		return modifyAmountRouteSensor;
	}
	public int getModifyAmountSwitchSetAI() {
		return modifyAmountSwitchSet;
	}

	public String getModelVariant() {
		return modelVariant;
	}
	public void setModelVariant(String modelVariant) {
		this.modelVariant = modelVariant;
	}
	
	public int getModelSize() {
		return modelSize;
	}
	public void setModelSize(int modelSize) {
		this.modelSize = modelSize;
	}

	public int getInferencing() {
		return inferencing;
	}
	public void setInferencing(int inferencing) {
		this.inferencing = inferencing;
	}

	public long getTimeout() {
		return timeout;
	}
	public void setTimeout(long timeout) {
		this.timeout = timeout;
	}

	public boolean isCreateSandbox() {
		return createSandbox;
	}
	public void setCreateSandbox(boolean createSandbox) {
		this.createSandbox = createSandbox;
	}

	public boolean isBenchmarkMode() {
		return benchmarkMode;
	}
	public void setBenchmarkMode(boolean benchmarkMode) {
		this.benchmarkMode = benchmarkMode;
	}


	public String getProjectPath() {
		// as long as this is in a file
		return RuntimeConstants.projectPath;
	}

	public String getCurrentScenario() {
		return currentScenario;
	}

	public void setCurrentScenario(String currentScenario) {
		this.currentScenario = currentScenario;
	}

	// userIterate
	public int getUserIterate() {
		return userIterate;
	}
	public void setUserIterate(int userIterate) {
		this.userIterate = userIterate;
	}

	// divPossiblyBad
	public int getDivPossiblyBad() {
		return divPossiblyBad;
	}
	public void setDivPossiblyBad(int divPossiblyBad) {
		this.divPossiblyBad = divPossiblyBad;
	}

	public Collection<Object[]> getSizes() {
		return sizes;
	}

	public void setSizes(Collection<Object[]> sizes) {
		this.sizes = sizes;
	}

	public String getInstanceModelPath() {
		return instanceModelPath;
	}

	public void setInstanceModelPath(String instanceModelPath) {
		this.instanceModelPath = instanceModelPath;
	}

	public String getPathProject_InstanceModels() {
		return pathProject_InstanceModels;
	}

	public void setPathProject_InstanceModels(String pathProject_InstanceModels) {
		this.pathProject_InstanceModels = pathProject_InstanceModels;
	}

	public String getPathWorkspace() {
		return pathWorkspace;
	}

	public void setPathWorkspace(String pathWorkspace) {
		this.pathWorkspace = pathWorkspace;
	}

	public boolean isNeoIndex() {
		return neoIndex;
	}

	public void setNeoIndex(boolean neoIndex) {
		this.neoIndex = neoIndex;
	}

	public String getQueryVariant() {
		return queryVariant;
	}

	public void setQueryVariant(String queryVariant) {
		this.queryVariant = queryVariant;
	}

	public String getInstanceVariant() {
		return instanceVariant;
	}

	public void setInstanceVariant(String instanceVariant) {
		this.instanceVariant = instanceVariant;
	}

	public String getFunctionalDependencyOption() {
		return functionalDependencyOption;
	}

	public void setFunctionalDependencyOption(String functionalDependencyOption) {
		this.functionalDependencyOption = functionalDependencyOption;
	}

	public String getStubTrimOption() {
		return stubTrimOption;
	}

	public void setStubTrimOption(String stubTrimOption) {
		this.stubTrimOption = stubTrimOption;
	}

	public int getScenarioVariant() {
		return scenarioVariant;
	}

	public void setScenarioVariant(int scenarioVariant) {
		this.scenarioVariant = scenarioVariant;
	}

}
