package hu.bme.mit.train.util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Map;
import org.eclipse.emf.ecore.resource.Resource;

import hu.bme.mit.train.generator.util.XMIResourceFactoryImplWithUUID;

public class Util {
	
	public static void runGC() {
		for (int i = 0; i < 5; ++i) {
			Runtime.getRuntime().gc();
		}
	}
	
	public static void freeCache(BenchmarkConfig bc) {
		String clearCacheCommand = bc.getProjectPath() + "/scripts/clearCache.sh";
		try {
			Process child;
			
		    String command = "/bin/sync";
		    child = Runtime.getRuntime().exec(command);
		    child.waitFor();
		    if (child.exitValue() != 0)
		    	throw new InterruptedException("Bad return value: " + child.exitValue());
		    
		    child = Runtime.getRuntime().exec("sudo -n " + clearCacheCommand);
		    child.waitFor();
		    if (child.exitValue() != 0)
		    	throw new InterruptedException("Bad return value: " + child.exitValue());
		} catch (IOException e) {
			e.printStackTrace(); // should never happen
		} catch (InterruptedException e) {
			System.err.println("Maybe running " + clearCacheCommand + " as root is not permitted?");
			System.err.println("Add it to the /etc/sudoers file without asking password e.g.:");
			System.err.println("bence ALL=(ALL) NOPASSWD: " + clearCacheCommand);
			e.printStackTrace();
		}
	}

	public static String readFile( String file ) throws IOException {
		BufferedReader reader = new BufferedReader( new FileReader (file));
		String line  = null;
		StringBuilder stringBuilder = new StringBuilder();
		String ls = System.getProperty("line.separator");
		while( ( line = reader.readLine() ) != null ) {
			stringBuilder.append( line );
			stringBuilder.append( ls );
		}
		reader.close();
		return stringBuilder.toString();
	}
	
	public static void writeFile(String filename, String content) {
		BufferedWriter out;
		try {
			out = new BufferedWriter(new FileWriter(filename));
			out.write(content);
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}


	public static void sleepLocal(long ms) {
		try {
			Thread.sleep(ms);
		} catch (InterruptedException e1) {
			System.err.println("Can not sleep.");
			e1.printStackTrace();
		}
	}
	
	public static void registerUUIDXMIResourceFactory() {
		// Register the XMI resource factory for the .concept extension
		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		Map<String, Object> m = reg.getExtensionToFactoryMap();
		m.put("concept", new XMIResourceFactoryImplWithUUID());
	}

	public static boolean isFailedPrevTest(BenchmarkConfig bc, String name) {
		File file = new File(bc.getProjectPath() + "/runtimeInfo/" + name);
		return file.exists();
	}

	public static void failedTest(BenchmarkConfig bc, String name, Throwable e) {
		Writer writer = new StringWriter();
		PrintWriter printWriter = new PrintWriter(writer);
		e.printStackTrace(printWriter);
		try {
			writer.append(bc.toString());
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		writeFile(bc.getProjectPath() + "/runtimeInfo/" + name, writer.toString());
	}

	public static void startTest(BenchmarkConfig bc, String name) {
		writeFile(bc.getProjectPath() + "/runtimeInfo/" + name, "Running, or somehow killed!");
	}

	public static void doneTest(BenchmarkConfig bc, String name) {
		File file = new File(bc.getProjectPath() + "/runtimeInfo/" + name);
		file.delete();
	}
	
	public static int calcModify(BenchmarkConfig bc, int nModify, BenckmarkResult bmr) {
		Integer elementSize = ModificationConstants.modelSizeGet(bc.getCurrentScenario(), bc.getModelSize());
		Integer minElementSize = ModificationConstants.modelSizeGet(bc.getCurrentScenario(), 1);
		if (bc.getModifyMethod() == 2) {
			return elementSize / nModify;
		} else if ((bc.getModifyMethod() == 1) || (bc.getModifyMethod() == 0)) { 
			return nModify;
		} else if (bc.getModifyMethod() == 4){ 
			return new Long(Math.round(elementSize.doubleValue() / minElementSize.doubleValue() * new Double(nModify))).intValue();
		} else if (bc.getModifyMethod() == 8){ 
			return bmr.getInvalids().get(0).getValue().intValue() / nModify;
		} else if (bc.getModifyMethod() == 16){ 
			String testCase = bmr.getId().split("_")[0];
			return ModificationConstants.getNumberOfPossiblyBadElements(bc.getCurrentScenario(), testCase, bc.getModelSize()) / nModify;
		}
		return nModify;
	}
	
}
